@@ -34,7 +34,7 @@ import java.util.Properties;
   * object, which is not reusable or shareable.<p>
   * The Controller is capable of comparing whether nodes are in document order;
   * therefore it acts as a NodeOrderComparer.
-  * @author Michael H. Kay (mhkay@iclway.co.uk)
+  * @author Michael H. Kay
   */
 
 public class Controller extends Transformer implements NodeOrderComparer {
@@ -729,18 +729,23 @@ public class Controller extends Transformer implements NodeOrderComparer {
         b.setLineNumbering(lineNumbering);
         b.setErrorListener(errorListener);
 
-        Stripper stripper;
-        if (styleSheetElement==null) {
-            stripper = new Stripper(new Mode());
-        } else {
-            stripper = styleSheetElement.newStripper();
-        }
-		stripper.setController(this);
-		b.setStripper(stripper);
-
+        Stripper stripper = makeStripper(b);
         return b;
     }
 
+    public Stripper makeStripper(Builder b) {
+        Stripper s;
+        if (styleSheetElement==null) {
+            s = new Stripper(new Mode());
+        } else {
+            s = styleSheetElement.newStripper();
+        }
+        s.setController(this);
+        s.setUnderlyingEmitter(b);
+		b.setStripper(s);
+        return s;
+    }
+
     //////////////////////////////////////////////////////////////////////
     // Methods for handling decimal-formats
     //////////////////////////////////////////////////////////////////////
@@ -1263,7 +1268,7 @@ public class Controller extends Transformer implements NodeOrderComparer {
 // The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay of International Computers Limited (mhkay@iclway.co.uk).
+// Michael Kay
 //
 // Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
