@@ -1,18 +1,14 @@
 package com.icl.saxon;
-import com.icl.saxon.om.*;
-import com.icl.saxon.tree.TreeBuilder;
-
+import org.w3c.dom.*;
 import org.xml.sax.*;
 import org.xml.sax.ext.LexicalHandler;
 import org.xml.sax.helpers.AttributesImpl;
-import org.xml.sax.helpers.NamespaceSupport;
 import org.xml.sax.helpers.DefaultHandler;
+import org.xml.sax.helpers.NamespaceSupport;
 
 import java.util.Enumeration;
-import java.util.Locale;
 import java.util.Hashtable;
-
-import org.w3c.dom.*;
+import java.util.Locale;
 
 /**
 * DOMDriver.java: (pseudo-)SAX driver for DOM.<BR>
@@ -37,14 +33,14 @@ public class DOMDriver implements Locator, XMLReader
     private Hashtable nsDeclarations = new Hashtable();
     protected Node root = null;
     protected String systemId;
-    
+
     /**
     * Set the content handler.
     * @param handler The object to receive content events. If this also implements LexicalHandler,
     * it will also be notified of comments.
     */
-    
-    public void setContentHandler (ContentHandler handler) 
+
+    public void setContentHandler (ContentHandler handler)
     {
         this.contentHandler = handler;
         if (handler instanceof LexicalHandler) {
@@ -61,7 +57,7 @@ public class DOMDriver implements Locator, XMLReader
     * only locales using the English language are supported.
     * @param locale The locale for which diagnostics will be generated
     */
-    
+
     public void setLocale (Locale locale) throws SAXException
     {}
 
@@ -70,7 +66,7 @@ public class DOMDriver implements Locator, XMLReader
      * <b>SAX2</b>: Returns the object used when resolving external
      * entities during parsing (both general and parameter entities).
      */
-     
+
     public EntityResolver getEntityResolver ()
     {
 	    return null;
@@ -78,9 +74,9 @@ public class DOMDriver implements Locator, XMLReader
 
     /**
      * <b>SAX1, SAX2</b>: Set the entity resolver for this parser.
-     * @param handler The object to receive entity events.
+     * @param resolver The object to receive entity events.
      */
-     
+
     public void setEntityResolver (EntityResolver resolver) {}
 
 
@@ -88,7 +84,7 @@ public class DOMDriver implements Locator, XMLReader
      * <b>SAX2</b>: Returns the object used to process declarations related
      * to notations and unparsed entities.
      */
-     
+
     public DTDHandler getDTDHandler () {
         return null;
     }
@@ -112,21 +108,21 @@ public class DOMDriver implements Locator, XMLReader
      *
      * @param handler The object to receive document events.
      */
-     
+
     public void setDocumentHandler (DocumentHandler handler) {}
 
     /**
      * <b>SAX1, SAX2</b>: Set the error handler for this parser.
      * @param handler The object to receive error events.
      */
-     
+
     public void setErrorHandler (ErrorHandler handler) {}
 
     /**
      * <b>SAX2</b>: Returns the object used to receive callbacks for XML
      * errors of all levels (fatal, nonfatal, warning); this is never null;
      */
-     
+
     public ErrorHandler getErrorHandler () { return null; }
 
 
@@ -161,7 +157,6 @@ public class DOMDriver implements Locator, XMLReader
 
     /**
     * Walk a document (traversing the nodes depth first)
-    * @param doc The (DOM) Document object to walk.
     * @exception SAXException On any error in the document
     */
 
@@ -176,7 +171,7 @@ public class DOMDriver implements Locator, XMLReader
 
         contentHandler.setDocumentLocator(this);
         contentHandler.startDocument();
-        if (root instanceof Element) {
+        if (root.getNodeType() == Node.ELEMENT_NODE) {
             sendElement((Element)root);
         } else {
             walkNode(root);                         // walk the root node
@@ -196,7 +191,7 @@ public class DOMDriver implements Locator, XMLReader
         while (true) {
             gatherNamespaces(node, true);
             Node parent = node.getParentNode();
-            if (parent instanceof Element) {
+            if (parent.getNodeType() == Node.ELEMENT_NODE) {
                 node = (Element)parent;
             } else {
                 break;
@@ -204,16 +199,16 @@ public class DOMDriver implements Locator, XMLReader
         }
         outputElement(startNode, topAtts);
     }
-    
+
   /**
     * Walk an element of a document (traversing the children depth first)
-    * @param elem The DOM Element object to walk
+    * @param node The DOM Element object to walk
     * @exception SAXException On any error in the document
-    * 
+    *
     */
-    
+
     private void walkNode (Node node) throws SAXException
-    {        
+    {
         if (node.hasChildNodes()) {
             NodeList nit = node.getChildNodes();
             for (int i=0; i<nit.getLength(); i++) {
@@ -226,13 +221,13 @@ public class DOMDriver implements Locator, XMLReader
                         nsSupport.pushContext();
                         attlist.clear();
                         nsDeclarations.clear();
-                        
+
                         // we can't rely on namespace declaration attributes being present -
                         // there may be undeclared namespace prefixes. (If the DOM is a Saxon
                         // tree, there will be no namespace declaration attributes.) So we
                         // declare all namespaces encountered, to be on the safe side.
-                        
-                        String elname = element.getTagName();
+
+                        //String elname = element.getTagName();
                         //if (elname.indexOf(':')>=0) {
                                   // also need to do this for default namespace
                             try {
@@ -247,8 +242,8 @@ public class DOMDriver implements Locator, XMLReader
                             } catch (Throwable err) {
                                 // it must be a level 1 DOM
                             }
-                        //}                            
-                         
+                        //}
+
                         NamedNodeMap atts = element.getAttributes();
                         for (int a1=0; a1<atts.getLength(); a1++) {
                             Attr att = (Attr)atts.item(a1);
@@ -282,12 +277,12 @@ public class DOMDriver implements Locator, XMLReader
                         		    }
                                 } catch (Throwable err) {
                                     // it must be a level 1 DOM
-                                }                            		
-                            }                                
+                                }
+                            }
                         }
                         for (int a2=0; a2<atts.getLength(); a2++) {
                             Attr att = (Attr)atts.item(a2);
-                            String attname = att.getName(); 
+                            String attname = att.getName();
                             if (!attname.equals("xmlns") && !attname.startsWith("xmlns:")) {
                                 //System.err.println("Processing attribute " + attname);
                                 String[] parts2 = nsSupport.processName(attname, parts, true);
@@ -307,7 +302,7 @@ public class DOMDriver implements Locator, XMLReader
                         String raw = elparts2[2];
 
                         contentHandler.startElement(uri, local, raw, attlist);
-                        
+
                         walkNode(element);
 
                         contentHandler.endElement(uri, local, raw);
@@ -333,7 +328,7 @@ public class DOMDriver implements Locator, XMLReader
                                 lexicalHandler.comment(text.toCharArray(), 0, text.length());
                             }
                         }
-                        break;    
+                        break;
                     case Node.TEXT_NODE:
                     case Node.CDATA_SECTION_NODE:
                         String text = ((CharacterData)child).getData();
@@ -349,8 +344,8 @@ public class DOMDriver implements Locator, XMLReader
                 }
             }
         }
-        
-    }                   
+
+    }
 
     private void outputElement(Element element, NamedNodeMap atts) throws SAXException {
         String[] elparts2 = nsSupport.processName(element.getTagName(), elparts, false);
@@ -360,10 +355,10 @@ public class DOMDriver implements Locator, XMLReader
         String uri = elparts2[0];
         String local = elparts2[1];
         String raw = elparts2[2];
-      
+
         for (int a2=0; a2<atts.getLength(); a2++) {
             Attr att = (Attr)atts.item(a2);
-            String attname = att.getName(); 
+            String attname = att.getName();
             if (!attname.equals("xmlns") && !attname.startsWith("xmlns:")) {
                 //System.err.println("Processing attribute " + attname);
                 String[] parts2 = nsSupport.processName(attname, parts, true);
@@ -376,7 +371,7 @@ public class DOMDriver implements Locator, XMLReader
         }
 
         contentHandler.startElement(uri, local, raw, attlist);
-        
+
         walkNode(element);
 
         contentHandler.endElement(uri, local, raw);
@@ -449,7 +444,7 @@ public class DOMDriver implements Locator, XMLReader
         }
         return atts;
     }
-    
+
     //
     // Implementation of org.xml.sax.Locator.
     //
@@ -489,7 +484,7 @@ public class DOMDriver implements Locator, XMLReader
      * @exception SAXNotRecognizedException thrown if the feature flag
      *	is neither built in, nor yet assigned.
      */
-     
+
     public boolean getFeature (String featureId) throws SAXNotRecognizedException
     {
 	    if ((FEATURE + "validation").equals (featureId))
@@ -521,7 +516,7 @@ public class DOMDriver implements Locator, XMLReader
      * @exception SAXNotRecognizedException thrown if the property value
      *	is neither built in, nor yet stored.
      */
-     
+
     public Object getProperty (String name) throws SAXNotRecognizedException {
         if (name.equals("http://xml.org/sax/properties/lexical-handler")) {
             return lexicalHandler;
@@ -536,7 +531,7 @@ public class DOMDriver implements Locator, XMLReader
      * motable.
      */
     public void setFeature (String featureId, boolean on)
-    throws SAXNotRecognizedException, SAXNotSupportedException 
+    throws SAXNotRecognizedException, SAXNotSupportedException
     {
 	    if ((FEATURE + "validation").equals (featureId))
 	        if (on) {
@@ -579,18 +574,18 @@ public class DOMDriver implements Locator, XMLReader
 	        }
 
 	    throw new SAXNotRecognizedException("Feature not recognized: " + featureId);
-    }    
+    }
 
     /**
      * <b>SAX2</b>:  Assigns the specified property.  Like SAX1 handlers,
      * these may be changed at any time.
      */
-     
+
     public void setProperty (String propertyId, Object property)
     throws SAXNotRecognizedException, SAXNotSupportedException {
         if (propertyId.equals("http://xml.org/sax/properties/lexical-handler")) {
             if (property instanceof LexicalHandler) {
-                lexicalHandler = (LexicalHandler)property;     
+                lexicalHandler = (LexicalHandler)property;
             } else {
                 throw new SAXNotSupportedException(
                     "Lexical Handler must be instance of org.xml.sax.ext.LexicalHandler");
@@ -604,18 +599,18 @@ public class DOMDriver implements Locator, XMLReader
 //
 // The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
 // you may not use this file except in compliance with the License. You may obtain a copy of the
-// License at http://www.mozilla.org/MPL/ 
+// License at http://www.mozilla.org/MPL/
 //
 // Software distributed under the License is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
-// See the License for the specific language governing rights and limitations under the License. 
+// See the License for the specific language governing rights and limitations under the License.
 //
-// The Original Code is: all this file. 
+// The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay of International Computers Limited (mhkay@iclway.co.uk).
+// Michael Kay
 //
-// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
+// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
-// Contributor(s): none. 
+// Contributor(s): none.
 //
