@@ -20,7 +20,7 @@ import java.util.Properties;
   * Only the transformation language is implemented (not the formatting objects).
   * Saxon extensions are documented in the file extensions.html
   *
-  * @author M.H.Kay (mhkay@iclway.co.uk)
+  * @author M.H.Kay
   */
 
 public class StyleSheet  {
@@ -248,6 +248,10 @@ public class StyleSheet  {
 
             if (useURLs || sourceFileName.startsWith("http:") || sourceFileName.startsWith("file:")) {
                 sourceInput = factory.getURIResolver().resolve(sourceFileName, null);
+                if (sourceInput == null) {
+                    // try the standard URI resolver
+                    sourceInput = new StandardURIResolver(factory).resolve(sourceFileName, null);
+                }
 
             } else {
                 sourceFile = new File(sourceFileName);
@@ -294,6 +298,10 @@ public class StyleSheet  {
                 if (useURLs || styleFileName.startsWith("http:")
                                  || styleFileName.startsWith("file:")) {
                     styleSource = factory.getURIResolver().resolve(styleFileName, null);
+                    if (styleSource == null) {
+                        // try the standard URI resolver
+                        sourceInput = new StandardURIResolver(factory).resolve(styleFileName, null);
+                    }
 
                 } else {
                     File sheetFile = new File(styleFileName);
@@ -571,7 +579,7 @@ public class StyleSheet  {
 // The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay of International Computers Limited (mhkay@iclway.co.uk).
+// Michael Kay
 //
 // Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
