@@ -1,16 +1,15 @@
 package com.icl.saxon.functions;
-import com.icl.saxon.*;
+import com.icl.saxon.Context;
+import com.icl.saxon.Controller;
+import com.icl.saxon.expr.*;
 import com.icl.saxon.om.*;
+import com.icl.saxon.pattern.AnyNodeTest;
 import com.icl.saxon.tinytree.TinyBuilder;
 import com.icl.saxon.tree.AttributeCollection;
-import com.icl.saxon.expr.*;
-import com.icl.saxon.output.*;
-import com.icl.saxon.pattern.AnyNodeTest;
-import com.icl.saxon.om.Axis;
+
 import javax.xml.transform.TransformerException;
-import java.util.*;
-import java.text.*;
-import org.w3c.xsl.XSLTContext;
+import java.util.StringTokenizer;
+import java.util.Vector;
 
 /**
 * This class implements functions that are supplied as standard with SAXON,
@@ -716,7 +715,7 @@ public class Extensions  {
 // The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay of International Computers Limited (mhkay@iclway.co.uk).
+// Michael Kay
 //
 // Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
