@@ -1,30 +1,22 @@
 package com.icl.saxon.jdom;
-import com.icl.saxon.om.NodeInfo;
-import com.icl.saxon.om.DocumentInfo;
-import com.icl.saxon.om.AxisEnumeration;
-import com.icl.saxon.om.NamePool;
-import com.icl.saxon.om.Axis;
-import com.icl.saxon.pattern.NodeTest;
-import com.icl.saxon.pattern.AnyNodeTest;
+import com.icl.saxon.om.*;
 import com.icl.saxon.output.Outputter;
-import com.icl.saxon.om.EmptyEnumeration;
-import com.icl.saxon.om.SingletonEnumeration;
+import com.icl.saxon.pattern.AnyNodeTest;
+import com.icl.saxon.pattern.NodeTest;
 import org.jdom.*;
+import org.jdom.Namespace;
+
 import javax.xml.transform.TransformerException;
-
-// NB: this module requires JDK 1.2
-
-import java.util.List;
-import java.util.Iterator;
-import java.util.ListIterator;
-import java.util.Stack;
 import java.util.HashMap;
+import java.util.Iterator;
+import java.util.List;
+import java.util.ListIterator;
 
 /**
   * A node in the XML parse tree representing an XML element, character content, or attribute.<P>
   * This is the top class in the interface hierarchy for nodes; see NodeImpl for the implementation
   * hierarchy.
-  * @author <A HREF="mailto:mhkay@iclway.co.uk>Michael H. Kay</A> 
+  * @author Michael H. Kay
   */
 
 public class NodeWrapper implements NodeInfo {
@@ -33,18 +25,18 @@ public class NodeWrapper implements NodeInfo {
     protected short nodeType;
     protected NodeWrapper parent;
     protected DocumentWrapper docWrapper;
-    protected int index;    
+    protected int index;
         // the index is the only way of distinguishing
         // two text nodes with the same parent!
-    
+
     public NodeWrapper(Object node, NodeWrapper parent, int index) {
         this.node = node;
         this.parent = parent;
         this.index = index;
     }
-    
+
     public NodeWrapper makeWrapper(Object node, NodeWrapper parent, int index) {
-        NodeWrapper wrapper; 
+        NodeWrapper wrapper;
         if (node instanceof Document) {
             return docWrapper;
         } else if (node instanceof Element) {
@@ -65,17 +57,20 @@ public class NodeWrapper implements NodeInfo {
         } else if (node instanceof ProcessingInstruction) {
             wrapper = new NodeWrapper(node, parent, index);
             wrapper.nodeType = NodeInfo.PI;
+        } else if (node instanceof Namespace) {
+            wrapper = new NodeWrapper(node, parent, index);
+            wrapper.nodeType = NodeInfo.NAMESPACE;
         } else {
             throw new IllegalArgumentException("Bad node type in JDOM! " + node.getClass() + " instance " + node.toString());
         }
         wrapper.docWrapper = parent.docWrapper;
         return wrapper;
-    }                           
+    }
 
     /**
     * Get the underlying JDOM node
     */
-    
+
     public Object getNode() {
         return node;
     }
@@ -88,7 +83,7 @@ public class NodeWrapper implements NodeInfo {
     public short getNodeType() {
         return nodeType;
     }
-    
+
     /**
     * Determine whether this is the same node as another node. <br />
     * Note: a.isSameNode(b) if and only if generateId(a)==generateId(b)
@@ -96,7 +91,7 @@ public class NodeWrapper implements NodeInfo {
     * same node in the tree.
     */
 
-    public boolean isSameNode(NodeInfo other) {
+    public boolean isSameNodeInfo(NodeInfo other) {
         if (!(other instanceof NodeWrapper)) {
             return false;
         }
@@ -108,7 +103,7 @@ public class NodeWrapper implements NodeInfo {
             return false;
         }
         if (node instanceof String) {
-            return parent.isSameNode(ow.parent);
+            return parent.isSameNodeInfo(ow.parent);
         } else {
             return node.equals(ow.node);    // this test doesn't work for text nodes in beta 0.7
                                             // but it's OK in beta 0.8
@@ -125,19 +120,19 @@ public class NodeWrapper implements NodeInfo {
     public String getSystemId() {
         return docWrapper.baseURI;
     }
-    
+
     public void setSystemId(String uri) {
         docWrapper.baseURI = uri;
     }
-    
+
     /**
     * Get the Base URI for the node, that is, the URI used for resolving a relative URI contained
     * in the node. This will be the same as the System ID unless xml:base has been used.
     */
-    
+
     public String getBaseURI() {
         return docWrapper.baseURI;
-    }        
+    }
 
     /**
     * Get line number
@@ -147,7 +142,7 @@ public class NodeWrapper implements NodeInfo {
     public int getLineNumber() {
         return -1;
     }
-    
+
     /**
     * Determine the relative position of this node and another node, in document order.
     * The other node will always be in the same document.
@@ -156,23 +151,23 @@ public class NodeWrapper implements NodeInfo {
     * node, or 0 if they are the same node. (In this case, isSameNode() will always
     * return true, and the two nodes will produce the same result for generateId())
     */
-    
+
     public int compareOrder(NodeInfo other) {
 
         NodeWrapper ow = (NodeWrapper)other;
-                
+
         // are they the same node?
-        if (this.isSameNode(other)) {
+        if (this.isSameNodeInfo(other)) {
             return 0;
         }
-        
+
         // are they siblings (common case)
-        if (this.getParent().isSameNode(other.getParent())) {
+        if (this.getParent().isSameNodeInfo(other.getParent())) {
             return this.index - ow.index;
         }
-        
+
         // find the depths of both nodes in the tree
-        
+
         int depth1 = 0;
         int depth2 = 0;
         NodeInfo p1 = this;
@@ -184,14 +179,14 @@ public class NodeWrapper implements NodeInfo {
         while (p2 != null) {
             depth2++;
             p2 = p2.getParent();
-        }        
+        }
 
         // move up one branch of the tree so we have two nodes on the same level
 
         p1 = this;
         while (depth1>depth2) {
             p1 = p1.getParent();
-            if (p1.isSameNode(ow)) {
+            if (p1.isSameNodeInfo(ow)) {
                 return +1;
             }
             depth1--;
@@ -200,11 +195,11 @@ public class NodeWrapper implements NodeInfo {
         p2 = ow;
         while (depth2>depth1) {
             p2 = p2.getParent();
-            if (p2.isSameNode(this)) {
+            if (p2.isSameNodeInfo(this)) {
                 return -1;
             }
             depth2--;
-        }            
+        }
 
         // now move up both branches in sync until we find a common parent
         while (true) {
@@ -213,14 +208,14 @@ public class NodeWrapper implements NodeInfo {
             if (par1==null || par2==null) {
                 throw new NullPointerException("JDOM tree compare - internal error");
             }
-            if (par1.isSameNode(par2)) {
+            if (par1.isSameNodeInfo(par2)) {
                 return ((NodeWrapper)p1).index - ((NodeWrapper)p2).index;
             }
             p1 = par1;
             p2 = par2;
         }
     }
-    
+
     /**
     * Return the string value of the node. The interpretation of this depends on the type
     * of node. For an element it is the accumulated character content of the element,
@@ -235,36 +230,36 @@ public class NodeWrapper implements NodeInfo {
                 StringBuffer sb1 = new StringBuffer();
                 expandStringValue(children1, sb1);
                 return sb1.toString();
-                
+
             case NodeInfo.ELEMENT:
                 List children2 = ((Element)node).getContent();
                 StringBuffer sb2 = new StringBuffer();
                 expandStringValue(children2, sb2);
                 return sb2.toString();
-                
+
             case NodeInfo.ATTRIBUTE:
                 return ((Attribute)node).getValue();
-                
+
             case NodeInfo.TEXT:
                 if (node instanceof String) return (String)node;
                 if (node instanceof Text) return ((Text)node).getText();
                 if (node instanceof CDATA) return ((CDATA)node).getText();
                 return "";
-                 
+
             case NodeInfo.COMMENT:
                 return ((Comment)node).getText();
-                
+
             case NodeInfo.PI:
                 return ((ProcessingInstruction)node).getData();
-                
+
             case NodeInfo.NAMESPACE:
                 return ((Namespace)node).getURI();
-                
+
             default:
                 return "";
         }
     }
-    
+
     private static void expandStringValue(List list, StringBuffer sb) {
         Iterator iter = list.iterator();
         while (iter.hasNext()) {
@@ -281,19 +276,22 @@ public class NodeWrapper implements NodeInfo {
                 throw new IllegalStateException("Unexpanded entity in JDOM tree");
             }
         }
-    }        
+    }
 
 	/**
 	* Get name code. The name code is a coded form of the node name: two nodes
 	* with the same name code have the same namespace URI, the same local name,
 	* and the same prefix. By masking the name code with &0xfffff, you get a
 	* fingerprint: two nodes with the same fingerprint have the same local name
-	* and namespace URI. 
+	* and namespace URI.
     * @see com.icl.saxon.om.NamePool#allocate allocate
 	*/
-	
+
 	public int getNameCode() {
 	    switch (nodeType) {
+            case NAMESPACE:
+                if (((Namespace)node).getPrefix().equals("")) return -1;
+                // else fall through
 	        case ELEMENT:
 	        case ATTRIBUTE:
 	        case PI:
@@ -311,7 +309,7 @@ public class NodeWrapper implements NodeInfo {
 	* with the same name code have the same namespace URI and the same local name.
 	* A fingerprint of -1 should be returned for a node with no name.
 	*/
-	
+
 	public int getFingerprint() {
 	    return getNameCode()&0xfffff;
 	}
@@ -338,7 +336,7 @@ public class NodeWrapper implements NodeInfo {
             default:
                 return "";
         }
-    }        
+    }
 
     /**
     * Get the prefix part of the name of this node. This is the name before the ":" if any.
@@ -354,7 +352,7 @@ public class NodeWrapper implements NodeInfo {
             default:
                 return "";
         }
-    }        
+    }
 
     /**
     * Get the URI part of the name of this node. This is the URI corresponding to the
@@ -372,12 +370,12 @@ public class NodeWrapper implements NodeInfo {
             default:
                 return "";
         }
-    }              
+    }
 
     /**
     * Get the display name of this node. For elements and attributes this is [prefix:]localname.
     * For unnamed nodes, it is an empty string.
-    * @return The display name of this node. 
+    * @return The display name of this node.
     * For a node with no name, return an empty string.
     */
 
@@ -394,19 +392,19 @@ public class NodeWrapper implements NodeInfo {
                 return "";
 
         }
-    }               
+    }
 
     /**
     * Get the NodeInfo object representing the parent of this node
     */
-    
+
     public NodeInfo getParent() {
         return parent;
     }
 
     /**
     * Return an enumeration over the nodes reached by the given axis from this node
-    * @param nodeType the type(s) of node to be included, e.g. NodeInfo.ELEMENT, NodeInfo.TEXT.
+    * @param nodeTest the type(s) of node to be included, e.g. NodeInfo.ELEMENT, NodeInfo.TEXT.
     * The value NodeInfo.NODE means include any type of node.
     * @param nodeTest A pattern to be matched by the returned nodes
     * @return a NodeEnumeration that scans the nodes reached by the axis in turn.
@@ -419,19 +417,19 @@ public class NodeWrapper implements NodeInfo {
                 return new FilterEnumeration(
                             new AncestorEnumeration(this, false),
                             nodeTest);
-                 
-            case Axis.ANCESTOR_OR_SELF: 
+
+            case Axis.ANCESTOR_OR_SELF:
                 if (nodeType==ROOT) return EmptyEnumeration.getInstance();
                 return new FilterEnumeration(
                             new AncestorEnumeration(this, true),
                             nodeTest);
-                 
+
             case Axis.ATTRIBUTE:
                 if (nodeType!=ELEMENT) return EmptyEnumeration.getInstance();
                 return new FilterEnumeration(
                             new AttributeEnumeration(this),
                             nodeTest);
-                 
+
             case Axis.CHILD:
                 if (hasChildNodes()) {
                     return new FilterEnumeration(
@@ -440,7 +438,7 @@ public class NodeWrapper implements NodeInfo {
                 } else {
                     return EmptyEnumeration.getInstance();
                 }
-                 
+
             case Axis.DESCENDANT:
                 if (hasChildNodes()) {
                     return new FilterEnumeration(
@@ -449,17 +447,17 @@ public class NodeWrapper implements NodeInfo {
                 } else {
                     return EmptyEnumeration.getInstance();
                 }
-                 
+
             case Axis.DESCENDANT_OR_SELF:
                  return new FilterEnumeration(
                             new DescendantEnumeration(this, true, true),
                             nodeTest);
-                 
+
             case Axis.FOLLOWING:
                  return new FilterEnumeration(
                             new FollowingEnumeration(this),
                             nodeTest);
-                 
+
             case Axis.FOLLOWING_SIBLING:
                  switch (nodeType) {
                     case ROOT:
@@ -471,7 +469,7 @@ public class NodeWrapper implements NodeInfo {
                             new ChildEnumeration(this, false, true),
                             nodeTest);
                  }
-                 
+
             case Axis.NAMESPACE:
                  if (nodeType!=ELEMENT) return EmptyEnumeration.getInstance();
                  return new FilterEnumeration(
@@ -479,17 +477,17 @@ public class NodeWrapper implements NodeInfo {
                                 nodeTest);
                  //throw new IllegalArgumentException(
                  //           "namespace axis not implemented for JDOM");
-                    
+
             case Axis.PARENT:
                  if (parent==null) return EmptyEnumeration.getInstance();
                  if (nodeTest.matches(parent)) return new SingletonEnumeration(parent);
                  return EmptyEnumeration.getInstance();
-                 
+
             case Axis.PRECEDING:
                  return new FilterEnumeration(
                             new PrecedingEnumeration(this, false),
                             nodeTest);
-                                             
+
             case Axis.PRECEDING_SIBLING:
                  switch (nodeType) {
                     case ROOT:
@@ -501,45 +499,47 @@ public class NodeWrapper implements NodeInfo {
                             new ChildEnumeration(this, false, false),
                             nodeTest);
                  }
-                 
+
             case Axis.SELF:
                  if (nodeTest.matches(this)) return new SingletonEnumeration(this);
                  return EmptyEnumeration.getInstance();
-                 
+
             case Axis.PRECEDING_OR_ANCESTOR:
                  return new FilterEnumeration(
                             new PrecedingEnumeration(this, true),
                             nodeTest);
-                                             
+
             default:
                  throw new IllegalArgumentException("Unknown axis number " + axisNumber);
         }
-    }            
-         
+    }
+
     /**
      * Find the value of a given attribute of this node. <BR>
      * This method is defined on all nodes to meet XSL requirements, but for nodes
      * other than elements it will always return null.
      * @param uri the namespace uri of an attribute ("" if no namespace)
-     * @param localname the local name of the attribute
+     * @param localName the local name of the attribute
      * @return the value of the attribute, if it exists, otherwise null
      */
 
     public String getAttributeValue(String uri, String localName) {
         if (nodeType==NodeInfo.ELEMENT) {
-            Namespace ns = Namespace.getNamespace(uri);
+            Namespace ns = (uri.equals("http://www.w3.org/XML/1998/namespace") ?
+                            Namespace.XML_NAMESPACE :
+                            Namespace.getNamespace(uri));
             return ((Element)node).getAttributeValue(localName, ns);
         } else {
             return "";
         }
-    } 
-   
+    }
+
     /**
     * Get the value of a given attribute of this node
     * @param fingerprint The fingerprint of the attribute name
     * @return the attribute value if it exists or null if not
     */
-    
+
     public String getAttributeValue(int fingerprint) {
         if (nodeType==NodeInfo.ELEMENT) {
             Iterator list = ((Element)node).getAttributes().iterator();
@@ -553,9 +553,9 @@ public class NodeWrapper implements NodeInfo {
                     return att.getValue();
                 }
             }
-        }            
+        }
         return null;
-    }         
+    }
 
     /**
     * Get the root (document) node
@@ -581,7 +581,7 @@ public class NodeWrapper implements NodeInfo {
             default:
                 return false;
         }
-    }               
+    }
     /**
     * Get a character string that uniquely identifies this node.<br />
     * Note: a.isSameNode(b) if and only if generateId(a)==generateId(b)
@@ -596,17 +596,17 @@ public class NodeWrapper implements NodeInfo {
         } else {
             return "j" + node.hashCode();
         }
-    } 
+    }
 
     /**
     * Copy this node to a given outputter (deep copy)
     */
 
     public void copy(Outputter out) throws TransformerException {
-        
+
         // TODO: this is a completely generic implementation: it ought to invoke a
         // helper class.
-        
+
         switch (nodeType) {
             case NodeInfo.ROOT:
                 AxisEnumeration children0 = getEnumeration(Axis.CHILD, new AnyNodeTest());
@@ -614,53 +614,55 @@ public class NodeWrapper implements NodeInfo {
                     children0.nextElement().copy(out);
                 }
                 return;
-                
+
             case NodeInfo.ELEMENT:
         		int nc = getNameCode();
                 out.writeStartTag(nc);
 
                 // output the namespaces
-                
+
                 outputNamespaceNodes(out, true);
-        
+
                 // output the attributes
 
                 AxisEnumeration attributes = getEnumeration(Axis.ATTRIBUTE, new AnyNodeTest());
                 while (attributes.hasMoreElements()) {
                     attributes.nextElement().copy(out);
                 }
-        
+
                 // output the children
-        
+
                 AxisEnumeration children = getEnumeration(Axis.CHILD, new AnyNodeTest());
                 while (children.hasMoreElements()) {
                     children.nextElement().copy(out);
                 }
-                
+
                 // finally the end tag
-        
+
                 out.writeEndTag(nc);
-                return;        
-                
+                return;
+
             case NodeInfo.ATTRIBUTE:
                 out.writeAttribute(getNameCode(), getStringValue());
                 return;
-                
+
             case NodeInfo.TEXT:
                 out.writeContent(getStringValue());
                 return;
-                
+
             case NodeInfo.COMMENT:
                 out.writeComment(getStringValue());
                 return;
-                
+
             case NodeInfo.PI:
                 out.writePI(getLocalName(), getStringValue());
                 return;
-                
+
             case NodeInfo.NAMESPACE:
-                throw new IllegalArgumentException("Copying Namespace node!");
-                
+                out.copyNamespaceNode(docWrapper.getNamePool().allocateNamespaceCode(
+                        ((Namespace)node).getPrefix(),
+                        ((Namespace)node).getURI()));
+
             default:
 
         }
@@ -687,10 +689,10 @@ public class NodeWrapper implements NodeInfo {
             // TODO: use the includeAncestors flag for optimization
         if (nodeType==ELEMENT) {
             NamePool pool = docWrapper.getNamePool();
-            AxisEnumeration enum = getEnumeration(Axis.NAMESPACE, 
+            AxisEnumeration enm = getEnumeration(Axis.NAMESPACE,
                                                   AnyNodeTest.getInstance());
-            while (enum.hasMoreElements()) {
-                Namespace ns = (Namespace)((NodeWrapper)enum.nextElement()).node;
+            while (enm.hasMoreElements()) {
+                Namespace ns = (Namespace)((NodeWrapper)enm.nextElement()).node;
                 int nscode = pool.allocateNamespaceCode(
                                 ns.getPrefix(),
                                 ns.getURI());
@@ -702,13 +704,13 @@ public class NodeWrapper implements NodeInfo {
     ///////////////////////////////////////////////////////////////////////////////
     // Axis enumeration classes
     ///////////////////////////////////////////////////////////////////////////////
-    
+
     private class FilterEnumeration implements AxisEnumeration {
         private BaseEnumeration base;
         private NodeTest nodeTest;
         private NodeInfo next;
         private int last = -1;
-        
+
         public FilterEnumeration(BaseEnumeration base, NodeTest test) {
             this.base = base;
             this.nodeTest = test;
@@ -722,23 +724,23 @@ public class NodeWrapper implements NodeInfo {
     	    }
     	    next = null;
     	}
-    	
+
     	public NodeInfo nextElement() {
     	    NodeInfo n = next;
     	    advance();
     	    return n;
         }
-            	
+
     	public boolean hasMoreElements() {
     	    return next!=null;
     	}
-    	
+
     	public int getLastPosition() {
-    	    
+
     	    // To find out how many nodes there are in the axis, we
     	    // make a copy of the original node enumeration, and run through
     	    // the whole thing again, counting how many nodes match the filter.
-    	    
+
     	    if (last>=0) {
     	        return last;
     	    }
@@ -754,7 +756,7 @@ public class NodeWrapper implements NodeInfo {
     	}
 
     	public boolean isSorted() {
-    	    return base.isSorted();         
+    	    return base.isSorted();
     	}
 
         public boolean isReverseSorted() {
@@ -765,7 +767,7 @@ public class NodeWrapper implements NodeInfo {
     	    return base.isPeer();
     	}
 	}
-	
+
 	private abstract class BaseEnumeration implements AxisEnumeration {
 
         protected NodeWrapper next;
@@ -773,7 +775,7 @@ public class NodeWrapper implements NodeInfo {
         public final boolean hasMoreElements() {
             return next!=null;
         }
-        
+
         public final NodeInfo nextElement() {
             NodeInfo n = next;
             advance();
@@ -781,32 +783,32 @@ public class NodeWrapper implements NodeInfo {
         }
 
         public abstract void advance();
-	    
+
 	    public abstract BaseEnumeration copy();
-	    
+
 	    public final int getLastPosition() {// not used
-	        return 1;   
+	        return 1;
 	    }
-	    
+
 	    public boolean isSorted() {
 	        return true;
 	    }
-	    
+
 	    public final boolean isReverseSorted() {
 	        return !isSorted();
 	    }
-	    
+
 	    public boolean isPeer() {
 	        return false;
 	    }
-	}        
-        
+	}
+
 
     final class AncestorEnumeration extends BaseEnumeration {
-    
+
         private boolean includeSelf;
         private NodeWrapper start;
-    
+
         public AncestorEnumeration(NodeWrapper start, boolean includeSelf) {
             this.start = start;
             this.includeSelf = includeSelf;
@@ -823,29 +825,29 @@ public class NodeWrapper implements NodeInfo {
 	    public boolean isSorted() {
 	        return false;
 	    }
-	    
+
 	    public boolean isPeer() {
 	        return false;
 	    }
-	        
+
         public BaseEnumeration copy() {
             return new AncestorEnumeration(start, includeSelf);
         }
-    
+
     } // end of class AncestorEnumeration
 
     private final class AttributeEnumeration extends BaseEnumeration {
-    
+
         private Iterator atts;
         private int ix = 0;
         private NodeWrapper start;
-    
+
         public AttributeEnumeration(NodeWrapper start) {
             this.start = start;
             atts = ((Element)start.node).getAttributes().iterator();
             advance();
         }
-           
+
         public void advance() {
             if (atts.hasNext()) {
                 next = makeWrapper(atts.next(), start, ix++);
@@ -853,34 +855,34 @@ public class NodeWrapper implements NodeInfo {
                 next = null;
             }
         }
-        
+
 	    public boolean isPeer() {
 	        return true;
 	    }
-	    
+
         public BaseEnumeration copy() {
             return new AttributeEnumeration(start);
         }
-        
+
     }  // end of class AttributeEnumeration
 
     private final class NamespaceEnumeration extends BaseEnumeration {
-    
-        private HashMap nslist = new HashMap();          
+
+        private HashMap nslist = new HashMap();
         private Iterator prefixes;
         private int ix = 0;
         private NodeWrapper start;
-    
+
         public NamespaceEnumeration(NodeWrapper start) {
             this.start = start;
             NodeWrapper curr = start;
-            
+
             // build the complete list of namespaces
 
             do {
                 Element elem = (Element)curr.node;
                 Namespace ns = elem.getNamespace();
-                if (nslist.containsKey(ns.getPrefix())) {
+                if (!nslist.containsKey(ns.getPrefix())) {
                     nslist.put(ns.getPrefix(), ns);
                 }
                 List addl = elem.getAdditionalNamespaces();
@@ -888,18 +890,19 @@ public class NodeWrapper implements NodeInfo {
                     Iterator itr = addl.iterator();
                     while (itr.hasNext()) {
                         ns = (Namespace) itr.next();
-                        if (nslist.containsKey(ns.getPrefix())) {
+                        if (!nslist.containsKey(ns.getPrefix())) {
                             nslist.put(ns.getPrefix(), ns);
                         }
                     }
                 }
                 curr = (NodeWrapper)curr.getParent();
             } while (curr.getNodeType()==ELEMENT);
-            
+
             nslist.put("xml", Namespace.XML_NAMESPACE);
-            prefixes = nslist.keySet().iterator();                
+            prefixes = nslist.keySet().iterator();
+            advance();
         }
-        
+
         public void advance() {
             if (prefixes.hasNext()) {
                 String prefix = (String)prefixes.next();
@@ -917,10 +920,10 @@ public class NodeWrapper implements NodeInfo {
         public BaseEnumeration copy() {
             return new NamespaceEnumeration(start);
         }
-        
+
     }  // end of class NamespaceEnumeration
 
-    
+
     /**
     * The class ChildEnumeration handles not only the child axis, but also the
     * following-sibling and preceding-sibling axes. It can also enumerate the children
@@ -929,15 +932,15 @@ public class NodeWrapper implements NodeInfo {
     */
 
     private class ChildEnumeration extends BaseEnumeration {
-        
+
         private NodeWrapper start;
         private NodeWrapper commonParent;
         private ListIterator children;
         private int ix = 0;
         private boolean downwards;  // enumerate children of start node (not siblings)
         private boolean forwards;   // enumerate in document order (not reverse order)
-        
-        public ChildEnumeration(NodeWrapper start, 
+
+        public ChildEnumeration(NodeWrapper start,
                                 boolean downwards, boolean forwards) {
             this.start = start;
             this.downwards = downwards;
@@ -956,7 +959,7 @@ public class NodeWrapper implements NodeInfo {
             }
 
             if (downwards) {
-                if (!forwards) {    
+                if (!forwards) {
                     // backwards enumeration: go to the end
                     while (children.hasNext()) {
                         children.next();
@@ -967,7 +970,7 @@ public class NodeWrapper implements NodeInfo {
                 ix = start.index;
                 // find the start node among the list of siblings
                 if (forwards) {
-                    for (int i=0; i<=ix; i++) { 
+                    for (int i=0; i<=ix; i++) {
                         children.next();
                     }
                     ix++;
@@ -980,17 +983,22 @@ public class NodeWrapper implements NodeInfo {
             }
             advance();
         }
-        
 
-            
+
+
         public void advance() {
             if (forwards) {
                 if (children.hasNext()) {
-                    Object nextChild = children.next();
-                    if (nextChild instanceof EntityRef) {
-                        throw new IllegalStateException("Unexpanded entity in JDOM tree");
-                    } else {
-                        next = makeWrapper(nextChild, commonParent, ix++);
+                    while (true) {
+                        Object nextChild = children.next();
+                        if (nextChild instanceof EntityRef) {
+                            throw new IllegalStateException("Unexpanded entity in JDOM tree");
+                        } else if (nextChild instanceof DocType) {
+                            continue;
+                        } else {
+                            next = makeWrapper(nextChild, commonParent, ix++);
+                            break;
+                        }
                     }
                 } else {
                     next = null;
@@ -1005,22 +1013,22 @@ public class NodeWrapper implements NodeInfo {
                     }
                 } else {
                     next = null;
-                }            
+                }
             }
         }
-    
+
         public boolean isSorted() {
             return forwards;
         }
-    
+
         public boolean isPeer() {
-            return true;           
+            return true;
         }
 
         public BaseEnumeration copy() {
             return new ChildEnumeration(start, downwards, forwards);
         }
-    
+
     } // end of class ChildEnumeration
 
     /**
@@ -1029,25 +1037,25 @@ public class NodeWrapper implements NodeInfo {
     * this is used when evaluating the preceding axis. Note that the includeSelf option
     * should not be used when scanning in reverse order, as the self node will always be
     * returned first.
-    */ 
+    */
 
     private final class DescendantEnumeration extends BaseEnumeration {
-        
+
         private AxisEnumeration children = null;
         private AxisEnumeration descendants = null;
         private NodeWrapper start;
         private boolean includeSelf;
         private boolean forwards;
         private boolean atEnd = false;
-    
-        public DescendantEnumeration(NodeWrapper start, 
+
+        public DescendantEnumeration(NodeWrapper start,
                                  boolean includeSelf, boolean forwards) {
             this.start = start;
             this.includeSelf = includeSelf;
             this.forwards = forwards;
             advance();
         }
-    
+
         public void advance() {
             if (descendants!=null) {
                 if (descendants.hasMoreElements()) {
@@ -1098,28 +1106,28 @@ public class NodeWrapper implements NodeInfo {
                 }
             }
         }
-    
+
         public boolean isSorted() {
             return forwards;
         }
-    
+
         public boolean isPeer() {
-            return false;           
+            return false;
         }
-    
+
         public BaseEnumeration copy() {
             return new DescendantEnumeration(start, includeSelf, forwards);
-        } 
-    
+        }
+
     } // end of class DescendantEnumeration
 
     private class FollowingEnumeration extends BaseEnumeration {
-        
+
         private NodeWrapper start;
         private AxisEnumeration ancestorEnum = null;
         private AxisEnumeration siblingEnum = null;
         private AxisEnumeration descendEnum = null;
-    
+
         public FollowingEnumeration(NodeWrapper start) {
             this.start = start;
             ancestorEnum = new AncestorEnumeration(start, false);
@@ -1140,7 +1148,7 @@ public class NodeWrapper implements NodeInfo {
             }
             advance();
         }
-            
+
         public void advance() {
             if (descendEnum!=null) {
                 if (descendEnum.hasMoreElements()) {
@@ -1162,7 +1170,7 @@ public class NodeWrapper implements NodeInfo {
                 } else {
                     descendEnum = null;
                     siblingEnum = null;
-                }                
+                }
             }
             if (ancestorEnum.hasMoreElements()) {
                 next = (NodeWrapper)ancestorEnum.nextElement();
@@ -1174,31 +1182,31 @@ public class NodeWrapper implements NodeInfo {
                 advance();
             } else {
                 next = null;
-            }     
+            }
         }
-    
+
         public boolean isSorted() {
             return true;
         }
-    
+
         public boolean isPeer() {
-            return false;           
+            return false;
         }
-    
+
         public BaseEnumeration copy() {
             return new FollowingEnumeration(start);
-        }    
-    
+        }
+
     } // end of class FollowingEnumeration
 
     private class PrecedingEnumeration extends BaseEnumeration {
-        
+
         private NodeWrapper start;
         private AxisEnumeration ancestorEnum = null;
         private AxisEnumeration siblingEnum = null;
         private AxisEnumeration descendEnum = null;
         private boolean includeAncestors;
-    
+
         public PrecedingEnumeration(NodeWrapper start, boolean includeAncestors) {
             this.start = start;
             this.includeAncestors = includeAncestors;
@@ -1216,7 +1224,7 @@ public class NodeWrapper implements NodeInfo {
             }
             advance();
         }
-            
+
         public void advance() {
             if (descendEnum!=null) {
                 if (descendEnum.hasMoreElements()) {
@@ -1240,7 +1248,7 @@ public class NodeWrapper implements NodeInfo {
                 } else {
                     descendEnum = null;
                     siblingEnum = null;
-                }                
+                }
             }
             if (ancestorEnum.hasMoreElements()) {
                 next = (NodeWrapper)ancestorEnum.nextElement();
@@ -1254,21 +1262,21 @@ public class NodeWrapper implements NodeInfo {
                 }
             } else {
                 next = null;
-            }     
+            }
         }
-    
+
         public boolean isSorted() {
             return false;
         }
-    
+
         public boolean isPeer() {
-            return false;           
+            return false;
         }
-    
+
         public BaseEnumeration copy() {
             return new PrecedingEnumeration(start, includeAncestors);
-        }    
-    
+        }
+
     } // end of class PrecedingEnumeration
 
 }
@@ -1276,18 +1284,18 @@ public class NodeWrapper implements NodeInfo {
 //
 // The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
 // you may not use this file except in compliance with the License. You may obtain a copy of the
-// License at http://www.mozilla.org/MPL/ 
+// License at http://www.mozilla.org/MPL/
 //
 // Software distributed under the License is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
-// See the License for the specific language governing rights and limitations under the License. 
+// See the License for the specific language governing rights and limitations under the License.
 //
-// The Original Code is: all this file. 
+// The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay (mhkay@iclway.co.uk).
+// Michael H. Kay.
 //
-// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
+// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
-// Contributor(s): none. 
+// Contributor(s): none.
 //
