@@ -15,22 +15,7 @@ public abstract class Name {
     */
 
     public static boolean isNCName(String name) {
-
-        // This isn't 100% accurate, e.g. FF10 to FF19 are classified as digits in Java but not
-        // in XML; 00AA is a letter in Java but not in XML.  But it's close enough.
-
-        if (name.length()==0) return false;
-        char first = name.charAt(0);
-        if (!(first=='_' || Character.isLetter(first))) {
-            return false;
-        }
-        for (int i=1; i<name.length(); i++) {
-            char c = name.charAt(i);
-            if (!(c=='_' || c=='.' || c=='-' || Character.isLetter(c) || Character.isDigit(c))) {
-                return false;
-            }
-        }
-        return true;
+        return XMLChar.isValidNCName(name);
     }
 
     /**
@@ -84,7 +69,7 @@ public abstract class Name {
 // The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay of International Computers Limited (mhkay@iclway.co.uk).
+// Michael Kay
 //
 // Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
