@@ -21,15 +21,16 @@ import javax.xml.transform.stream.StreamResult;
 import javax.xml.transform.dom.DOMResult;
 import javax.xml.transform.sax.SAXResult;
 import java.io.*;
+import java.lang.reflect.Constructor;
 
 /**
   * This class allows output to be generated. It channels output requests to an
   * Emitter which does the actual writing.
-  * 
-  * @author <A HREF="mailto:mhkay@iclway.co.uk>Michael H. Kay</A>
+  *
+  * @author Michael H. Kay
   */
 
-  
+
 
 public class GeneralOutputter extends Outputter {
 
@@ -38,13 +39,13 @@ public class GeneralOutputter extends Outputter {
     private Writer writer;
     private OutputStream outputStream;
     private boolean closeAfterUse = false;
-    
+
     private int pendingStartTag = -1;
     private AttributeCollection pendingAttList;
-    
+
     private int[] pendingNSList = new int[20];
     private int pendingNSListSize = 0;
-    
+
     private boolean suppressAttributes = false;
 
 	public GeneralOutputter(NamePool pool) {
@@ -59,26 +60,42 @@ public class GeneralOutputter extends Outputter {
     * @param result Details of the new output destination
     */
 
-    public void setOutputDestination(Properties props, Result result) 
+    public void setOutputDestination(Properties props, Result result)
     throws TransformerException {
         setOutputProperties(props);
-        
-        Emitter emitter = makeEmitter(props, result);       
+
+        Emitter emitter = makeEmitter(props, result);
         emitter.setNamePool(namePool);
         emitter.setOutputProperties(props);
-                
+
         setEmitter(emitter);
 
         open();
-    }        
+    }
 
     /**
     * The following atrocious code is borrowed from Xalan, where it is commented simply:
     * // yuck.
     * The backslash variants added by MHK.
     */
-    
+
     public static String urlToFileName(String base) {
+        try {
+            // try JDK 1.4+ approach: return new File(new URI(base)).toString()
+            Class uriClass = Loader.getClass("java.net.URI");
+            Class[] uriArgTypes = {String.class};
+            Object[] uriArgValues = {base};
+            Constructor constructor = uriClass.getConstructor(uriArgTypes);
+            Object uriInstance = constructor.newInstance(uriArgValues);
+            Class[] fileArgTypes = {uriClass};
+            Object[] fileArgValues = {uriInstance};
+            constructor = File.class.getConstructor(fileArgTypes);
+            File file = (File)constructor.newInstance(fileArgValues);
+            return file.toString();
+
+        } catch (Exception err) {
+            // continue
+        }
         if(null != base) {
             if (base.startsWith("file:////")) {
                 base = base.substring(7);
@@ -95,11 +112,11 @@ public class GeneralOutputter extends Outputter {
             } else if (base.startsWith("file:\\\\\\")) {
                 base = base.substring(6);
             } else if (base.startsWith("file:\\\\")) {
-                base = base.substring(5); 
+                base = base.substring(5);
             } else if (base.startsWith("file:\\")) {
                 base = base.substring(5);
             }
-        } 
+        }
         if (File.separatorChar != '/') {
             base = base.replace('/', File.separatorChar);
         }
@@ -109,7 +126,7 @@ public class GeneralOutputter extends Outputter {
     /**
     * Create a new FileOutputStream, given a filename and a baseURI
     */
-   
+
     public static FileOutputStream makeFileOutputStream(  String baseURI,
                                                     String fileName,
                                                     boolean mkdirs)
@@ -117,7 +134,7 @@ public class GeneralOutputter extends Outputter {
     {
         try {
             File file = new File(fileName);
-    
+
             if (!file.isAbsolute()) {
                 String base = urlToFileName(baseURI);
                 if(null != base) {
@@ -125,7 +142,7 @@ public class GeneralOutputter extends Outputter {
                     file = new File(baseFile.getParent(), fileName);
                 }
             }
-    
+
             if (mkdirs) {
                 String dirStr = file.getParent();
                 if((null != dirStr) && (dirStr.length() > 0)) {
@@ -133,12 +150,12 @@ public class GeneralOutputter extends Outputter {
                     dir.mkdirs();
                 }
             }
-    
+
             FileOutputStream ostream = new FileOutputStream(file);
             return ostream;
         } catch (Exception err) {
             throw new TransformerException("Failed to create output file", err);
-        }   
+        }
   }
 
 
@@ -149,7 +166,7 @@ public class GeneralOutputter extends Outputter {
 
     public Emitter makeEmitter(Properties props, Result result)
     throws TransformerException {
-    	      
+
         Emitter emitter;
         if (result instanceof DOMResult) {
             Node resultNode = ((DOMResult)result).getNode();
@@ -176,7 +193,7 @@ public class GeneralOutputter extends Outputter {
                         throw new TransformerException("Cannot add to an existing Saxon document");
                     }
                 } else {
-                    // Non-Saxon DOM                                                           
+                    // Non-Saxon DOM
                     emitter = new DOMEmitter();
                     ((DOMEmitter)emitter).setNode(resultNode);
                 }
@@ -198,16 +215,16 @@ public class GeneralOutputter extends Outputter {
         	    proxy.setLexicalHandler(sr.getLexicalHandler());
         	}
         	emitter = proxy;
-        	
+
         } else if (result instanceof Emitter) {
         	emitter = (Emitter)result;
-        	
+
         } else if (result instanceof StreamResult) {
-        
+
             String method = props.getProperty(OutputKeys.METHOD);
             if (method==null) {
             	emitter = new UncommittedEmitter();
-            	
+
             } else if (method.equals("html")) {
                 emitter = new HTMLEmitter();
                 if (!"no".equals(props.getProperty(OutputKeys.INDENT))) {
@@ -228,22 +245,22 @@ public class GeneralOutputter extends Outputter {
                     CDATAFilter filter = new CDATAFilter();
                     filter.setUnderlyingEmitter(emitter);
                     emitter=filter;
-                }            
+                }
             } else if (method.equals("text")) {
                 emitter = new TEXTEmitter();
-    
+
             } else {
                 // TODO: externally supplied properties must be validated
                 int brace = method.indexOf('}');
                 String localName = method.substring(brace+1);
                 int colon = localName.indexOf(':');
                 localName = localName.substring(colon+1);
-                
+
                 if (localName.equals("fop")) {
                     // avoid an explicit external reference to avoid build problems
-                    // when FOP is not present on the class path 
+                    // when FOP is not present on the class path
                     emitter = Emitter.makeEmitter("com.icl.saxon.fop.FOPEmitter");
-                } else if (localName.equals("xhtml")) { 
+                } else if (localName.equals("xhtml")) {
                     emitter = new XHTMLEmitter();
                     if ("yes".equals(props.getProperty(OutputKeys.INDENT))) {
                         HTMLIndenter in = new HTMLIndenter();
@@ -255,10 +272,10 @@ public class GeneralOutputter extends Outputter {
                         CDATAFilter filter = new CDATAFilter();
                         filter.setUnderlyingEmitter(emitter);
                         emitter=filter;
-                    }       
+                    }
                 } else {
                     emitter = Emitter.makeEmitter(localName);
-                } 
+                }
             }
             if (emitter.usesWriter()) {
                 writer = getStreamWriter((StreamResult)result, props);
@@ -266,17 +283,17 @@ public class GeneralOutputter extends Outputter {
             } else {
                 outputStream = getOutputStream((StreamResult)result, props);
                 emitter.setOutputStream(outputStream);
-            }     
-        
+            }
+
         } else {
             throw new IllegalArgumentException("Unknown type of Result: " + result.getClass());
-        }        
+        }
 
 		// add a filter to remove duplicate namespaces
 		NamespaceEmitter ne = new NamespaceEmitter();
 		ne.setUnderlyingEmitter(emitter);
 		emitter = ne;
-		return emitter;        
+		return emitter;
     }
 
     /**
@@ -285,7 +302,7 @@ public class GeneralOutputter extends Outputter {
 
     private Writer getStreamWriter(StreamResult result, Properties props)
     throws TransformerException {
-            
+
         closeAfterUse = false;
         Writer writer = result.getWriter();
         if (writer==null) {
@@ -299,14 +316,14 @@ public class GeneralOutputter extends Outputter {
                     closeAfterUse = true;
                 }
             }
-	        
+
             CharacterSet charSet = CharacterSetFactory.getCharacterSet(props);
-	        
+
             String encoding = props.getProperty(OutputKeys.ENCODING);
             if (encoding==null) encoding = "UTF8";
             if (encoding.equalsIgnoreCase("utf-8")) encoding = "UTF8";
                  // needed for Microsoft Java VM
-            
+
 	        if (charSet instanceof PluggableCharacterSet) {
 	        	encoding = ((PluggableCharacterSet)charSet).getEncodingName();
 	        }
@@ -347,7 +364,7 @@ public class GeneralOutputter extends Outputter {
 
     private OutputStream getOutputStream(StreamResult result, Properties props)
     throws TransformerException {
-            
+
         closeAfterUse = false;
 
         OutputStream outputStream = ((StreamResult)result).getOutputStream();
@@ -360,15 +377,15 @@ public class GeneralOutputter extends Outputter {
                 closeAfterUse = true;
             }
         }
-        
+
         if (outputStream==null) {
             throw new TransformerException("This output method requires a binary output destination");
         }
-        
+
         return outputStream;
     }
 
-		
+
     /**
     * Set the emitter that will deal with this output
     */
@@ -376,7 +393,7 @@ public class GeneralOutputter extends Outputter {
     private void setEmitter(Emitter handler) {
         this.emitter = handler;
     }
-   
+
     public void reset() throws TransformerException {
         if (pendingStartTag != -1) flushStartTag();
     }
@@ -390,7 +407,7 @@ public class GeneralOutputter extends Outputter {
     }
 
     /**
-    * Produce literal output. This is written as is, without any escaping. 
+    * Produce literal output. This is written as is, without any escaping.
     * The method is provided for Java applications that wish to output literal HTML text.
     * It is not used by the XSL system, which always writes using specific methods such as
     * writeStartTag().
@@ -426,7 +443,7 @@ public class GeneralOutputter extends Outputter {
         s.getChars(0, len, charbuffer, 0);
         writeContent(charbuffer, 0, len);
     }
-    
+
     /**
     * Produce text content output. <BR>
     * Special characters are escaped using XML/HTML conventions if the output format
@@ -472,7 +489,7 @@ public class GeneralOutputter extends Outputter {
     /**
     * Output an element start tag. <br>
     * The actual output of the tag is deferred until all attributes have been output
-    * using writeAttribute(). 
+    * using writeAttribute().
     * @param nameCode The element name code
     */
 
@@ -485,12 +502,12 @@ public class GeneralOutputter extends Outputter {
         } else {
             suppressAttributes = false;
         }
-        
+
         if (pendingStartTag != -1) flushStartTag();
         pendingAttList.clear();
         pendingNSListSize = 0;
         pendingStartTag = nameCode;
-        
+
         // make sure there is a namespace declaration for this start tag
         //writeNamespaceDeclaration(namePool.allocateNamespaceCode(nameCode));
         // ***** now done by the NamespaceEmitter
@@ -503,7 +520,7 @@ public class GeneralOutputter extends Outputter {
 	* also checks that the attribute namespace has been declared, and declares it
 	* if not.
 	*/
-	
+
 	public int checkAttributePrefix(int nameCode) throws TransformerException {
 		int nscode = namePool.allocateNamespaceCode(nameCode);
         for (int i=0; i<pendingNSListSize; i++) {
@@ -527,14 +544,14 @@ public class GeneralOutputter extends Outputter {
         writeNamespaceDeclaration(nscode);
         return nameCode;
     }
-    		
+
     /**
     * Output a namespace declaration. <br>
     * This is added to a list of pending namespaces for the current start tag.
     * If there is already another declaration of the same prefix, this one is
     * ignored.
     * Note that unlike SAX2 startPrefixMapping(), this call is made AFTER writing the start tag.
-    * @param nscode The namespace code 
+    * @param nscode The namespace code
     * @throws TransformerException if there is no start tag to write to (created using writeStartTag),
     * or if character content has been written since the start tag was written.
     */
@@ -543,14 +560,14 @@ public class GeneralOutputter extends Outputter {
     throws TransformerException {
 
         if (suppressAttributes) return;
-        
+
         // System.err.println("Write namespace prefix=" + (nscode>>16) + " uri=" + (nscode&0xffff));
         if (pendingStartTag==-1) {
             throw new TransformerException("Cannot write a namespace declaration when there is no open start tag");
         }
 
         // elimination of namespaces already present on an outer element of the
-        // result tree is now done by the NamespaceEmitter. 
+        // result tree is now done by the NamespaceEmitter.
 
         // Ignore declarations whose prefix is duplicated for this element.
 
@@ -575,7 +592,7 @@ public class GeneralOutputter extends Outputter {
 	* Copy a namespace node to the current element node
 	* (Rules defined in XSLT 1.0 errata)
 	*/
-	
+
 	public void copyNamespaceNode(int nscode) throws TransformerException {
         if (pendingStartTag==-1) {
             throw new TransformerException("Cannot copy a namespace node when there is no containing element node");
@@ -583,7 +600,7 @@ public class GeneralOutputter extends Outputter {
         if (pendingAttList.getLength()>0) {
         	throw new TransformerException("Cannot copy a namespace node to an element after attributes have been added");
         }
-        
+
         // check for duplicates
         for (int i=0; i<pendingNSListSize; i++) {
         	if ((nscode>>16) == (pendingNSList[i]>>16)) {
@@ -597,7 +614,7 @@ public class GeneralOutputter extends Outputter {
         	}
         }
         writeNamespaceDeclaration(nscode);
-    }        		
+    }
 
     /**
     * It is possible for a single output element to use the same prefix to refer to different
@@ -615,11 +632,11 @@ public class GeneralOutputter extends Outputter {
     * Test whether there is an open start tag. This determines whether it is
     * possible to write an attribute node at this point.
     */
-    
+
     public boolean thereIsAnOpenStartTag() {
         return (pendingStartTag != -1);
     }
-    
+
     /**
     * Output an attribute value. <br>
     * This is added to a list of pending attributes for the current start tag, overwriting
@@ -659,9 +676,9 @@ public class GeneralOutputter extends Outputter {
             throw new TransformerException("Cannot write an attribute when there is no open start tag");
         }
 
-        pendingAttList.setAttribute(nameCode, 
+        pendingAttList.setAttribute(nameCode,
               (noEscape ? "NO-ESC" : "CDATA"),  // dummy attribute type to indicate no special chars
-              value); 
+              value);
 
     }
 
@@ -676,7 +693,7 @@ public class GeneralOutputter extends Outputter {
         if (pendingStartTag != -1) {
             flushStartTag();
         }
-        
+
         // write the end tag
         emitter.endElement(nameCode);
     }
@@ -724,12 +741,12 @@ public class GeneralOutputter extends Outputter {
     * Flush out a pending start tag
     */
 
-    protected void flushStartTag() throws TransformerException {    
+    protected void flushStartTag() throws TransformerException {
         emitter.startElement(pendingStartTag, pendingAttList,
         						 pendingNSList, pendingNSListSize);
         pendingNSListSize = 0;
         pendingStartTag = -1;
-            
+
     }
 
 }
@@ -737,18 +754,18 @@ public class GeneralOutputter extends Outputter {
 //
 // The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
 // you may not use this file except in compliance with the License. You may obtain a copy of the
-// License at http://www.mozilla.org/MPL/ 
+// License at http://www.mozilla.org/MPL/
 //
 // Software distributed under the License is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
-// See the License for the specific language governing rights and limitations under the License. 
+// See the License for the specific language governing rights and limitations under the License.
 //
-// The Original Code is: all this file. 
+// The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay of International Computers Limited (mhkay@iclway.co.uk).
+// Michael Kay
 //
-// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
+// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
-// Contributor(s): none. 
+// Contributor(s): none.
 //
