@@ -1,32 +1,34 @@
 package com.icl.saxon.style;
-import com.icl.saxon.tree.AttributeCollection;
-import com.icl.saxon.*;
-import com.icl.saxon.om.*;
-import com.icl.saxon.tree.ElementWithAttributes;
-import com.icl.saxon.tree.DocumentImpl;
-import com.icl.saxon.tree.NodeImpl;
+import com.icl.saxon.Binding;
+import com.icl.saxon.Context;
+import com.icl.saxon.Controller;
+import com.icl.saxon.PreparedStyleSheet;
 import com.icl.saxon.expr.*;
-import com.icl.saxon.pattern.Pattern;
+import com.icl.saxon.om.NamePool;
+import com.icl.saxon.om.Namespace;
+import com.icl.saxon.om.NamespaceException;
+import com.icl.saxon.om.NodeInfo;
 import com.icl.saxon.pattern.NoNodeTest;
-import com.icl.saxon.output.*;
-import com.icl.saxon.trace.*;  // e.g.
-
-import org.xml.sax.*;
+import com.icl.saxon.pattern.Pattern;
+import com.icl.saxon.trace.TraceListener;
+import com.icl.saxon.tree.DocumentImpl;
+import com.icl.saxon.tree.ElementWithAttributes;
+import com.icl.saxon.tree.NodeImpl;
 import org.w3c.dom.Node;
+import org.xml.sax.Locator;
 
-import java.util.*;
-import java.io.*;
-import java.text.*;
-
-import javax.xml.transform.stream.StreamResult;
-import javax.xml.transform.TransformerException;
 import javax.xml.transform.TransformerConfigurationException;
+import javax.xml.transform.TransformerException;
+import java.util.Enumeration;
+import java.util.Hashtable;
+import java.util.StringTokenizer;
+import java.util.Vector;
 
 /**
 * Abstract superclass for all element nodes in the stylesheet. <BR>
 * Note: this class implements Locator. The element
 * retains information about its own location in the stylesheet, which is useful when
-* an XSL error is found. 
+* an XSL error is found.
 */
 
 public abstract class StyleElement extends ElementWithAttributes
@@ -37,17 +39,17 @@ public abstract class StyleElement extends ElementWithAttributes
     private short[] excludedNamespaces = null;		// a list of URI codes
     protected String version = null;
     protected StaticContext staticContext = null;
-    protected TransformerConfigurationException validationError = null;	
+    protected TransformerConfigurationException validationError = null;
     protected int reportingCircumstances = REPORT_ALWAYS;
 
     // Conditions under which an error is to be reported
-    
-    public final static int REPORT_ALWAYS = 1;
-    public final static int REPORT_UNLESS_FORWARDS_COMPATIBLE = 2;
-    public final static int REPORT_IF_INSTANTIATED = 3;
-    
+
+    public static final int REPORT_ALWAYS = 1;
+    public static final int REPORT_UNLESS_FORWARDS_COMPATIBLE = 2;
+    public static final int REPORT_IF_INSTANTIATED = 3;
+
     /**
-    * Constructor 
+    * Constructor
     */
 
     public StyleElement() {}
@@ -71,14 +73,14 @@ public abstract class StyleElement extends ElementWithAttributes
         this.version = temp.version;
         this.root = temp.root;
         this.staticContext = temp.staticContext;
-        this.validationError = temp.validationError;        
+        this.validationError = temp.validationError;
         this.reportingCircumstances = temp.reportingCircumstances;
-    }       
+    }
 
 	/**
 	* Set a validation error
 	*/
-	
+
 	protected void setValidationError(TransformerException reason,
 	                                  int circumstances) {
 	    if (reason instanceof TransformerConfigurationException) {
@@ -119,7 +121,7 @@ public abstract class StyleElement extends ElementWithAttributes
 	/**
 	* Get the containing XSLStyleSheet element
 	*/
-	
+
 	public XSLStyleSheet getContainingStyleSheet() {
 		NodeImpl next = this;
 		while (!(next instanceof XSLStyleSheet)) {
@@ -129,7 +131,7 @@ public abstract class StyleElement extends ElementWithAttributes
 	}
 
     /**
-    * Get the import precedence of this stylesheet element. 
+    * Get the import precedence of this stylesheet element.
     */
 
     public int getPrecedence() {
@@ -139,7 +141,7 @@ public abstract class StyleElement extends ElementWithAttributes
 	/**
 	* Get the StandardNames object
 	*/
-	
+
 	public final StandardNames getStandardNames() {
 		DocumentImpl root = (DocumentImpl)getDocumentRoot();
 		return ((StyleNodeFactory)root.getNodeFactory()).getStandardNames();
@@ -154,8 +156,11 @@ public abstract class StyleElement extends ElementWithAttributes
         processAttributes();
         NodeImpl child = (NodeImpl)getFirstChild();
         while (child != null) {
-            if (child instanceof StyleElement) {
-                ((StyleElement)child).processAllAttributes();          
+            if (child instanceof XSLStyleSheet) {
+                ((XSLStyleSheet)child).compileError(child.getDisplayName() +
+                        " cannot appear as a child of another element");
+            } else if (child instanceof StyleElement) {
+                ((StyleElement)child).processAllAttributes();
             }
             child = (NodeImpl)child.getNextSibling();
         }
@@ -183,7 +188,7 @@ public abstract class StyleElement extends ElementWithAttributes
     * Check whether an unknown attribute is permitted.
     * @param nc The name code of the attribute name
     */
-    
+
     protected void checkUnknownAttribute(int nc) throws TransformerConfigurationException {
     	if (forwardsCompatibleModeIsEnabled()) {
     		// then unknown attributes are permitted and ignored
@@ -193,7 +198,7 @@ public abstract class StyleElement extends ElementWithAttributes
     	String elementURI = getURI();
     	int attributeFingerprint = nc & 0xfffff;
     	StandardNames sn = getStandardNames();
-    	
+
     	// allow xsl:extension-element-prefixes etc on an extension element
 
     	if (isInstruction() &&
@@ -204,13 +209,13 @@ public abstract class StyleElement extends ElementWithAttributes
     		  attributeFingerprint == sn.XSL_VERSION)) {
     		return;
     	}
-    	
+
     	if (attributeURI.equals("") || attributeURI.equals(Namespace.XSLT)) {
 			compileError("Attribute " + getNamePool().getDisplayName(nc) +
 				 " is not allowed on this element");
         }
-    }    		
-    
+    }
+
 
     /**
     * Set the attribute list for the element. This is called to process the attributes (note
@@ -223,8 +228,8 @@ public abstract class StyleElement extends ElementWithAttributes
 	/**
 	* Make an expression in the context of this stylesheet element
 	*/
-	
-	public Expression makeExpression(String expression)  
+
+	public Expression makeExpression(String expression)
 	throws TransformerConfigurationException {
 	    try {
     		return Expression.make(expression, staticContext);
@@ -237,22 +242,26 @@ public abstract class StyleElement extends ElementWithAttributes
 	/**
 	* Make a pattern in the context of this stylesheet element
 	*/
-	
-	public Pattern makePattern(String pattern) 
+
+	public Pattern makePattern(String pattern)
 	throws TransformerConfigurationException {
 	    try {
 		    return Pattern.make(pattern, staticContext);
         } catch(XPathException err) {
-            compileError(err);
-            return NoNodeTest.getInstance();
+            if (forwardsCompatibleModeIsEnabled()) {
+                return NoNodeTest.getInstance();
+            } else {
+                compileError(err);
+                return NoNodeTest.getInstance();
+            }
         }
 	}
 
 	/**
 	* Make an attribute value template in the context of this stylesheet element
 	*/
-	
-	public Expression makeAttributeValueTemplate(String expression) 
+
+	public Expression makeAttributeValueTemplate(String expression)
 	throws TransformerConfigurationException {
 	    try {
 		    return AttributeValueTemplate.make(expression, staticContext);
@@ -267,7 +276,7 @@ public abstract class StyleElement extends ElementWithAttributes
     * @param nc the name code of the attribute required
     */
 
-    protected void processExtensionElementAttribute(int nc) 
+    protected void processExtensionElementAttribute(int nc)
     throws TransformerConfigurationException {
         String ext = getAttributeValue(nc & 0xfffff);
         if (ext!=null) {
@@ -275,9 +284,9 @@ public abstract class StyleElement extends ElementWithAttributes
         	int count = 0;
             StringTokenizer st1 = new StringTokenizer(ext);
             while (st1.hasMoreTokens()) {
-                String s = st1.nextToken();
+                st1.nextToken();
                 count++;
-            }       
+            }
 			extensionNamespaces = new short[count];
 			count = 0;
             StringTokenizer st2 = new StringTokenizer(ext);
@@ -293,7 +302,7 @@ public abstract class StyleElement extends ElementWithAttributes
                     extensionNamespaces = null;
                     compileError(err.getMessage());
                 }
-            }            
+            }
         }
     }
 
@@ -302,7 +311,7 @@ public abstract class StyleElement extends ElementWithAttributes
     * @param nc the name code of the attribute required
     */
 
-    protected void processExcludedNamespaces(int nc) 
+    protected void processExcludedNamespaces(int nc)
     throws TransformerConfigurationException {
         String ext = getAttributeValue(nc & 0xfffff);
         if (ext!=null) {
@@ -310,9 +319,9 @@ public abstract class StyleElement extends ElementWithAttributes
         	int count = 0;
             StringTokenizer st1 = new StringTokenizer(ext);
             while (st1.hasMoreTokens()) {
-                String s = st1.nextToken();
+                st1.nextToken();
                 count++;
-            }       
+            }
 			excludedNamespaces = new short[count];
 			count = 0;
             StringTokenizer st2 = new StringTokenizer(ext);
@@ -328,7 +337,7 @@ public abstract class StyleElement extends ElementWithAttributes
                     excludedNamespaces = null;
                     compileError(err.getMessage());
                 }
-            }            
+            }
         }
     }
 
@@ -353,7 +362,7 @@ public abstract class StyleElement extends ElementWithAttributes
             } else {
                 version = "1.0";    // defensive programming
             }
-        } 
+        }
         return version;
     }
 
@@ -364,7 +373,7 @@ public abstract class StyleElement extends ElementWithAttributes
     public boolean forwardsCompatibleModeIsEnabled() {
         return !(getVersion().equals("1.0"));
     }
-        
+
     /**
     * Check whether a particular extension element namespace is defined on this node.
     * This checks this node only, not the ancestor nodes.
@@ -429,7 +438,7 @@ public abstract class StyleElement extends ElementWithAttributes
 
     public boolean isExcludedNamespace(short uriCode) {
 		if (uriCode==Namespace.XSLT_CODE) return true;
-        if (isExtensionNamespace(uriCode)) return true;        
+        if (isExtensionNamespace(uriCode)) return true;
         NodeImpl anc = this;
         while (anc instanceof StyleElement) {
             if (((StyleElement)anc).definesExcludedNamespace(uriCode)) {
@@ -469,21 +478,21 @@ public abstract class StyleElement extends ElementWithAttributes
                           && !forwardsCompatibleModeIsEnabled()) {
                 compileError(validationError);
             }
-        } else {
-            try {
-                validate();
-            } catch (TransformerConfigurationException err) {
-            	if (forwardsCompatibleModeIsEnabled()) {
-            		setValidationError(err, REPORT_IF_INSTANTIATED);
-            	} else {
-                	compileError(err);
-                }
-            }
-            
-            validateChildren();
         }
+        try {
+            validate();
+        } catch (TransformerConfigurationException err) {
+            if (forwardsCompatibleModeIsEnabled()) {
+                setValidationError(err, REPORT_IF_INSTANTIATED);
+            } else {
+                compileError(err);
+            }
+        }
+
+        validateChildren();
+
     }
-    
+
     protected void validateChildren() throws TransformerConfigurationException {
         NodeImpl child = (NodeImpl)getFirstChild();
         while (child != null) {
@@ -588,7 +597,7 @@ public abstract class StyleElement extends ElementWithAttributes
     */
 
     public void processChildren(Context context) throws TransformerException {
-        
+
     	if (context.getController().isTracing()) { // e.g.
     	    TraceListener listener = context.getController().getTraceListener();
 
@@ -610,7 +619,7 @@ public abstract class StyleElement extends ElementWithAttributes
 	        		    } catch (TransformerException err) {
 	        			    throw snode.styleError(err);
 	        		    }
-	        		}                    
+	        		}
         		}
 
         		listener.leave(node, context);
@@ -635,7 +644,7 @@ public abstract class StyleElement extends ElementWithAttributes
 	        		    } catch (TransformerException err) {
 	        			    throw snode.styleError(err);
 	        		    }
-	        		}                    
+	        		}
         		}
         		node = (NodeImpl)node.getNextSibling();
     	    }
@@ -643,17 +652,10 @@ public abstract class StyleElement extends ElementWithAttributes
     	}
     }
 
-    // diagnostic version of process()
-
-    private void processX(Context context) throws TransformerException {
-        System.err.println("Processing " + context.getCurrentNodeInfo() + " using " + this);
-        process(context);
-    }
-
-	/**
+    /**
 	* Perform fallback processing
 	*/
-	
+
 	protected void fallbackProcessing(StyleElement instruction, Context context) throws TransformerException {
         // process any xsl:fallback children; if there are none, report the original failure reason
         XSLFallback fallback = null;
@@ -665,7 +667,7 @@ public abstract class StyleElement extends ElementWithAttributes
             }
             child = child.getNextSibling();
         }
-      
+
         if (fallback==null) {
         	throw instruction.styleError(instruction.validationError);
         }
@@ -675,7 +677,7 @@ public abstract class StyleElement extends ElementWithAttributes
         while (child!=null) {
             if (child instanceof XSLFallback) {
                 XSLFallback f = (XSLFallback)child;
-                
+
                 if (tracing) {
                     TraceListener listener = context.getController().getTraceListener();
             		listener.enter(f, context);
@@ -687,8 +689,8 @@ public abstract class StyleElement extends ElementWithAttributes
             }
             child = child.getNextSibling();
         }
-		
-		
+
+
 	}
 
     /**
@@ -696,21 +698,29 @@ public abstract class StyleElement extends ElementWithAttributes
     * and XSLApplyTemplates
     */
 
-    protected Expression handleSortKeys(Expression select) {
+    protected Expression handleSortKeys(Expression select) throws TransformerConfigurationException {
         // handle sort keys if any
-        
+
         int numberOfSortKeys = 0;
+        boolean sortAllowed = true;
         Node child = getFirstChild();
 
         while(child!=null) {
             if (child instanceof XSLSort) {
-                numberOfSortKeys++;          
+                if (!sortAllowed) {
+                    compileError("An xsl:sort element is not allowed here");
+                }
+                numberOfSortKeys++;
+            } else {
+                if (!(this instanceof XSLApplyTemplates && child instanceof XSLWithParam)) {
+                    sortAllowed = false;
+                }
             }
             child = child.getNextSibling();
-        }        
+        }
 
         if (numberOfSortKeys > 0) {
-            SortedSelection sortExpression = new SortedSelection(select, numberOfSortKeys);  
+            SortedSelection sortExpression = new SortedSelection(select, numberOfSortKeys);
             child = getFirstChild();
 
             int k=0;
@@ -718,12 +728,12 @@ public abstract class StyleElement extends ElementWithAttributes
                 if (child instanceof XSLSort) {
                     sortExpression.setSortKey(
                         ((XSLSort)child).getSortKeyDefinition(),
-                        k++);               
+                        k++);
                 }
                 child = child.getNextSibling();
             }
             return sortExpression;
-        
+
         } else {
             return new NodeListExpression(select);  // sorts into document order
         }
@@ -735,11 +745,11 @@ public abstract class StyleElement extends ElementWithAttributes
     * result elements
     */
 
-    protected void findAttributeSets(String use) 
+    protected void findAttributeSets(String use)
     throws TransformerConfigurationException {
 
-        attributeSets = new Vector();
-        
+        attributeSets = new Vector(5);
+
         XSLStyleSheet stylesheet = getPrincipalStyleSheet();
         Vector toplevel = stylesheet.getTopLevel();
 
@@ -761,16 +771,16 @@ public abstract class StyleElement extends ElementWithAttributes
             for (int i=0; i<toplevel.size(); i++) {
                 if (toplevel.elementAt(i) instanceof XSLAttributeSet) {
                     XSLAttributeSet t = (XSLAttributeSet)toplevel.elementAt(i);
-                    if (t.getAttributeSetFingerprint() == fprint) {  
+                    if (t.getAttributeSetFingerprint() == fprint) {
                         attributeSets.addElement(t);
                         found = true;
                     }
                 }
             }
-	        
+
             if (!found) {
                 compileError("No attribute-set exists named " + asetname);
-            }            
+            }
         }
     }
 
@@ -778,18 +788,19 @@ public abstract class StyleElement extends ElementWithAttributes
     * Expand the attribute sets referenced in this element's use-attribute-sets attribute
     */
 
-    protected void processAttributeSets(Context context) 
+    protected void processAttributeSets(Context context)
     throws TransformerException {
         if (attributeSets==null) return;
         Controller c = context.getController();
         for (int i=0; i<attributeSets.size(); i++) {
             XSLAttributeSet aset = (XSLAttributeSet)attributeSets.elementAt(i);
-            
+
             // detect circular references. TODO: Ideally we should do this at compile time
+            // 2004-10-09: note patch in getUserData(), setUserData()
             Object isBeingExpanded = c.getUserData(aset, "is-being-expanded");
             if (isBeingExpanded!=null) {
                 throw styleError("Circular reference to attribute set");
-            } 
+            }
             c.setUserData(aset, "is-being-expanded", "is-being-expanded");
             aset.expand(context);
             c.setUserData(aset, "is-being-expanded", null);
@@ -819,10 +830,10 @@ public abstract class StyleElement extends ElementWithAttributes
     * Construct an exception with diagnostic information
     */
 
-    protected void compileError(TransformerException error) 
+    protected void compileError(TransformerException error)
     throws TransformerConfigurationException {
         if (error.getLocator()==null) {
-            error.setLocator(this);                                     
+            error.setLocator(this);
         }
         PreparedStyleSheet pss = getPreparedStyleSheet();
         try {
@@ -846,8 +857,8 @@ public abstract class StyleElement extends ElementWithAttributes
     }
 
     protected void compileError(String message)
-    throws TransformerConfigurationException { 
-        TransformerConfigurationException tce = 
+    throws TransformerConfigurationException {
+        TransformerConfigurationException tce =
             new TransformerConfigurationException(message);
         tce.setLocator(this);
         compileError(tce);
@@ -863,9 +874,9 @@ public abstract class StyleElement extends ElementWithAttributes
 
     /**
     * Bind a variable used in this element to the XSLVariable element in which it is declared
-    * @param fingerprint The fingerprint of the name of the variable 
+    * @param fingerprint The fingerprint of the name of the variable
     * @return a Binding for the variable
-    * @throw XPathException if the variable has not been declared
+    * @throws XPathException if the variable has not been declared
     */
 
     public Binding bindVariable(int fingerprint) throws XPathException {
@@ -878,7 +889,7 @@ public abstract class StyleElement extends ElementWithAttributes
 
     /**
     * Bind a variable used in this element to the XSLVariable element in which it is declared
-    * @name The absolute name of the variable (prefixed by namespace URI)
+    * @param fprint The absolute name of the variable (as a namepool fingerprint)
     * @return a Binding for the variable, or null if it has not been declared
     */
 
@@ -905,10 +916,10 @@ public abstract class StyleElement extends ElementWithAttributes
                 }
             }
         }
-       
+
         // Now check for a global variable
         // we rely on the search following the order of decreasing import precedence.
-        
+
         XSLStyleSheet root = getPrincipalStyleSheet();
         Vector toplevel = root.getTopLevel();
         for (int i=toplevel.size()-1; i>=0; i--) {
@@ -920,16 +931,16 @@ public abstract class StyleElement extends ElementWithAttributes
                 }
             }
         }
-        
+
         return null;
     }
 
     /**
-    * List the variables that are in scope for this stylesheet element. 
+    * List the variables that are in scope for this stylesheet element.
     * Designed for a debugger, not used by the processor.
     * @return two Enumeration of Strings, the global ones [0] and the local ones [1]
     */
-    
+
     public Enumeration[] getVariableNames() {  // e.g.
         Hashtable local = new Hashtable();
         Hashtable global = new Hashtable();
@@ -939,7 +950,7 @@ public abstract class StyleElement extends ElementWithAttributes
         NamePool pool = getNamePool();
 
         // first collect the local variable declarations
-        
+
         if (!isTopLevel()) {
             while (true) {
                 curr = (NodeImpl)curr.getPreviousSibling();
@@ -954,7 +965,7 @@ public abstract class StyleElement extends ElementWithAttributes
                 	int fprint = ((Binding)curr).getVariableFingerprint();
                 	String uri = pool.getURI(fprint);
                 	String lname = pool.getLocalName(fprint);
-                    String varname = uri + "^" + lname;
+                    String varname = uri + '^' + lname;
         		    if (local.get(varname)==null) {
         			    local.put(varname, varname);
         		    }
@@ -964,7 +975,7 @@ public abstract class StyleElement extends ElementWithAttributes
 
         // Now collect the global variables
         // we rely on the search following the order of increasing import precedence.
-        
+
         XSLStyleSheet root = getPrincipalStyleSheet();
         Vector toplevel = root.getTopLevel();
         for (int i=0; i<toplevel.size(); i++) {
@@ -973,7 +984,7 @@ public abstract class StyleElement extends ElementWithAttributes
             	int fprint = ((Binding)child).getVariableFingerprint();
             	String uri = pool.getURI(fprint);
             	String lname = pool.getLocalName(fprint);
-                String varname = uri + "^" + lname;
+                String varname = uri + '^' + lname;
         		if (local.get(varname)==null) {
         		    global.put(varname, varname);
         		}
@@ -996,7 +1007,7 @@ public abstract class StyleElement extends ElementWithAttributes
     public Function getStyleSheetFunction(int fingerprint) {
 
         // we rely on the search following the order of decreasing import precedence.
-        
+
         XSLStyleSheet root = getPrincipalStyleSheet();
         Vector toplevel = root.getTopLevel();
         for (int i=toplevel.size()-1; i>=0; i--) {
@@ -1013,28 +1024,28 @@ public abstract class StyleElement extends ElementWithAttributes
 
 	/** needed to satisfy Locator interface
 	*/
-	
+
 	//public int getColumnNumber() {
-	//	return -1; 
+	//	return -1;
 	//}
 }
 
 //
 // The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
 // you may not use this file except in compliance with the License. You may obtain a copy of the
-// License at http://www.mozilla.org/MPL/ 
+// License at http://www.mozilla.org/MPL/
 //
 // Software distributed under the License is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
-// See the License for the specific language governing rights and limitations under the License. 
+// See the License for the specific language governing rights and limitations under the License.
 //
-// The Original Code is: all this file. 
+// The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay of International Computers Limited (mhkay@iclway.co.uk).
+// Michael Kay
 //
-// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
+// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
-// Contributor(s): 
+// Contributor(s):
 // Portions marked "e.g." are from Edwin Glaser (edwin@pannenleiter.de)
 //
