@@ -1,17 +1,14 @@
 package com.icl.saxon.style;
-import com.icl.saxon.tree.AttributeCollection;
 import com.icl.saxon.*;
 import com.icl.saxon.om.*;
-import com.icl.saxon.handlers.*;
-import com.icl.saxon.expr.*;
-import com.icl.saxon.output.*;
+import com.icl.saxon.trace.TraceListener;
+import com.icl.saxon.tree.AttributeCollection;
 import com.icl.saxon.tree.NodeImpl;
-import com.icl.saxon.trace.*;  // e.g.
-import javax.xml.transform.*;
 
-import org.xml.sax.Attributes;
-import java.util.*;
-import java.io.*;
+import javax.xml.transform.TransformerConfigurationException;
+import javax.xml.transform.TransformerException;
+import java.util.Properties;
+import java.util.Vector;
 
 /**
 * An xsl:stylesheet or xsl:transform element in the stylesheet.<BR>
@@ -33,35 +30,35 @@ public class XSLStyleSheet extends StyleElement {
 
                 // the lowest precedence of any stylesheet imported by this one
     private int minImportPrecedence = 0;
-    
+
                 // the StyleSheet that included or imported this one; null for the principal stylesheet
     private XSLStyleSheet importer = null;
-    
+
                 // the PreparedStyleSheet object used to load this stylesheet
     private PreparedStyleSheet stylesheet;
-    
+
                 // the top-level elements in this logical stylesheet (after include/import)
     private Vector topLevel;
-    
-                // definitions of strip/preserve space action    
+
+                // definitions of strip/preserve space action
     private Mode stripperRules = null;
-    
-                // definitions of template rules    
+
+                // definitions of template rules
     private RuleManager ruleManager;
-    
+
                 // definitions of keys
     private KeyManager keyManager = new KeyManager();
-    
+
                 // definitions of decimal formats
     private DecimalFormatManager decimalFormatManager = new DecimalFormatManager();
-    
+
                 // definitions of preview elements
     private PreviewManager previewManager = null;
-    
+
                 // media type (MIME type) of principal output
     //private String mediaType;
-    
-                // namespace aliases 
+
+                // namespace aliases
     private int numberOfAliases = 0;
     private short[] aliasSCodes = new short[5];
     private short[] aliasRCodes = new short[5];
@@ -81,7 +78,7 @@ public class XSLStyleSheet extends StyleElement {
         stylesheet = sheet;
         ruleManager = new RuleManager(sheet.getNamePool());
     }
-    
+
     /**
     * Get the owning PreparedStyleSheet object
     */
@@ -102,7 +99,7 @@ public class XSLStyleSheet extends StyleElement {
     /**
     * Get the rules determining which nodes are to be stripped from the tree
     */
-    
+
     protected Mode getStripperRules() {
         if (stripperRules==null) {
             stripperRules = new Mode();
@@ -118,26 +115,26 @@ public class XSLStyleSheet extends StyleElement {
     public Stripper newStripper() {
         return new Stripper(stripperRules);
     }
-    
+
     /**
     * Determine whether this stylesheet does any whitespace stripping
     */
-    
+
     public boolean stripsWhitespace() {
         StandardNames sn = getStandardNames();
         for (int i=0; i<topLevel.size(); i++) {
             NodeInfo s = (NodeInfo)topLevel.elementAt(i);
             if (s.getFingerprint()==sn.XSL_STRIP_SPACE) {
-                return true;              
+                return true;
             }
         }
         return false;
-    }        
-    
+    }
+
     /**
     * Get the KeyManager which handles key definitions
     */
-    
+
     public KeyManager getKeyManager() {
         return keyManager;
     }
@@ -192,12 +189,12 @@ public class XSLStyleSheet extends StyleElement {
     public int getMinImportPrecedence() {
         return minImportPrecedence;
     }
-    
+
     /**
     * Set the minimum import precedence of this stylesheet, that is, the lowest precedence
     * of any stylesheet imported by this one
     */
-    
+
     public void setMinImportPrecedence(int precedence) {
         minImportPrecedence = precedence;
     }
@@ -282,11 +279,11 @@ public class XSLStyleSheet extends StyleElement {
 			} else if (f==sn.EXTENSION_ELEMENT_PREFIXES) {
         		//
 			} else if (f==sn.EXCLUDE_RESULT_PREFIXES) {
-        		//    		        		
+        		//
         	} else {
         		checkUnknownAttribute(nc);
         	}
-        }        
+        }
         if (version==null) {
             reportAbsence("version");
         }
@@ -303,7 +300,7 @@ public class XSLStyleSheet extends StyleElement {
     /**
     * Get the declared namespace alias for a given namespace URI code if there is one.
     * If there is more than one, we get the last.
-    * @param uriCode The code of the uri used in the stylesheet. 
+    * @param uriCode The code of the uri used in the stylesheet.
     * @return The code of the result uri to be used: return the stylesheet uri unchanged
     * if no alias is defined
     */
@@ -322,7 +319,7 @@ public class XSLStyleSheet extends StyleElement {
     /**
     * Validate this element
     */
-    
+
     public void validate() throws TransformerConfigurationException {
         if (validationError != null) {
             compileError(validationError);
@@ -344,18 +341,18 @@ public class XSLStyleSheet extends StyleElement {
         // process any xsl:include and xsl:import elements
 
         spliceIncludes();
-        
+
 
         // process the attributes of every node in the tree
 
         processAllAttributes();
 
         // collect any namespace aliases
-        
+
         collectNamespaceAliases();
 
         // Validate the whole logical style sheet (i.e. with included and imported sheets)
-                     
+
         validate();
         for (int i=0; i<topLevel.size(); i++) {
             Object node = topLevel.elementAt(i);
@@ -365,7 +362,7 @@ public class XSLStyleSheet extends StyleElement {
         }
 
         // Preprocess definitions of top-level elements.
-        
+
         for (int i=0; i<topLevel.size(); i++) {
             Object s = topLevel.elementAt(i);
             if (s instanceof StyleElement) {
@@ -373,7 +370,7 @@ public class XSLStyleSheet extends StyleElement {
                     ((StyleElement)s).preprocess();
                 } catch (TransformerConfigurationException err) {
                     ((StyleElement)s).compileError(err);
-                }                    
+                }
             }
         }
 
@@ -389,9 +386,9 @@ public class XSLStyleSheet extends StyleElement {
         topLevel = new Vector();
         minImportPrecedence = precedence;
         StyleElement previousElement = this;
-        
+
         NodeImpl child = (NodeImpl)getFirstChild();
-        
+
         while(child!=null) {
 
             if (child.getNodeType() == NodeInfo.TEXT) {
@@ -400,13 +397,13 @@ public class XSLStyleSheet extends StyleElement {
                     previousElement.compileError(
                         "No character data is allowed between top-level elements");
                 }
-                
+
             } else {
                 previousElement = (StyleElement)child;
                 if (child instanceof XSLGeneralIncorporate) {
                     XSLGeneralIncorporate xslinc = (XSLGeneralIncorporate)child;
                     xslinc.processAttributes();
-                
+
                     if (xslinc.isImport()) {
                         if (foundNonImport) {
                             xslinc.compileError("xsl:import elements must come first");
@@ -417,8 +414,8 @@ public class XSLStyleSheet extends StyleElement {
 
                     // get the included stylesheet. This follows the URL, builds a tree, and splices
                     // in any indirectly-included stylesheets.
-                
-                    XSLStyleSheet inc = 
+
+                    XSLStyleSheet inc =
                         xslinc.getIncludedStyleSheet(this, precedence);
                     if (inc==null) return;  // error has been reported
 
@@ -440,7 +437,7 @@ public class XSLStyleSheet extends StyleElement {
                     // include a stylesheet, any xsl:import elements in the included document are moved
                     // up in the including document to after any xsl:import elements in the including
                     // document.
-               
+
                     Vector incchildren = inc.topLevel;
                     for (int j=0; j<incchildren.size(); j++) {
                         StyleElement elem = (StyleElement)incchildren.elementAt(j);
@@ -457,7 +454,7 @@ public class XSLStyleSheet extends StyleElement {
                 } else {
                     foundNonImport = true;
                     topLevel.addElement(child);
-                }                
+                }
             }
             child = (NodeImpl)child.getNextSibling();
         }
@@ -467,8 +464,8 @@ public class XSLStyleSheet extends StyleElement {
 	/**
 	* Collect any namespace aliases
 	*/
-	
-	private void collectNamespaceAliases() {	
+
+	private void collectNamespaceAliases() {
         for (int i=0; i<topLevel.size(); i++) {
             Object node = topLevel.elementAt(i);
             if (node instanceof XSLNamespaceAlias) {
@@ -488,10 +485,10 @@ public class XSLStyleSheet extends StyleElement {
             }
         }
     }
-    
+
     protected boolean hasNamespaceAliases() {
     	return numberOfAliases>0;
-    }   
+    }
 
     /**
     * Process the attributes of every node in the stylesheet
@@ -507,7 +504,7 @@ public class XSLStyleSheet extends StyleElement {
                     ((StyleElement)s).processAllAttributes();
                 } catch (TransformerConfigurationException err) {
                     ((StyleElement)s).compileError(err);
-                }                    
+                }
             }
         }
     }
@@ -531,12 +528,12 @@ public class XSLStyleSheet extends StyleElement {
     * unexpanded, and no validation performed.
     */
 
-    public void gatherOutputProperties(Properties details) {       
+    public void gatherOutputProperties(Properties details) {
         Vector children = topLevel;
         for (int i=0; i<children.size(); i++) {
             Object s = children.elementAt(i);
             if (s instanceof XSLOutput) {
-                ((XSLOutput)s).gatherOutputProperties(details);        
+                ((XSLOutput)s).gatherOutputProperties(details);
             }
         }
     }
@@ -548,12 +545,12 @@ public class XSLStyleSheet extends StyleElement {
     */
 
     public void updateOutputProperties(Properties details, Context context)
-    throws TransformerException {       
+    throws TransformerException {
         Vector children = topLevel;
         for (int i=0; i<children.size(); i++) {
             Object s = children.elementAt(i);
             if (s instanceof XSLOutput) {
-                ((XSLOutput)s).updateOutputProperties(details, context);        
+                ((XSLOutput)s).updateOutputProperties(details, context);
             }
         }
     }
@@ -564,7 +561,7 @@ public class XSLStyleSheet extends StyleElement {
     * @throws TransformerException if a class is found but cannot
     * be loaded
     */
-    
+
     public Class getExternalJavaClass(String uri) throws TransformerException {
         Vector children = topLevel;
         if (!((Boolean)getPreparedStyleSheet().getTransformerFactory().
@@ -578,11 +575,11 @@ public class XSLStyleSheet extends StyleElement {
                 Class c = script.getJavaClass(uri);
                 if (c != null) {
                     return c;
-                }        
+                }
             }
         }
         return null;
-    }        
+    }
 
     /**
     * Process() is called once the source document is available. It activates those top-level
@@ -619,7 +616,7 @@ public class XSLStyleSheet extends StyleElement {
             if (s instanceof StyleElement) {
                 try {
                     if (tracing && !(s instanceof XSLTemplate)) {
-                        listener.enter((StyleElement)s, context);                                                    
+                        listener.enter((StyleElement)s, context);
                         ((StyleElement)s).process(context);
                         listener.leave((StyleElement)s, context);
                     } else {
@@ -628,30 +625,30 @@ public class XSLStyleSheet extends StyleElement {
                 } catch (TransformerException err) {
                     throw ((StyleElement)s).styleError(err);
                 }
-            } 
+            }
         }
     }
 
 
-    
+
 }
 
 //
 // The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
 // you may not use this file except in compliance with the License. You may obtain a copy of the
-// License at http://www.mozilla.org/MPL/ 
+// License at http://www.mozilla.org/MPL/
 //
 // Software distributed under the License is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
-// See the License for the specific language governing rights and limitations under the License. 
+// See the License for the specific language governing rights and limitations under the License.
 //
-// The Original Code is: all this file. 
+// The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay of International Computers Limited (mhkay@iclway.co.uk).
+// Michael Kay
 //
-// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
+// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
-// Contributor(s): 
+// Contributor(s):
 // Portions marked "e.g." are from Edwin Glaser (edwin@pannenleiter.de)
 //
