@@ -23,11 +23,11 @@ import javax.xml.transform.TransformerException;
   * This is the top-level class in the implementation class hierarchy; it essentially contains
   * all those methods that can be defined using other primitive methods, without direct access
   * to data.
-  * @author <A HREF="mailto:mhkay@iclway.co.uk>Michael H. Kay</A> 
+  * @author Michael H. Kay
   */
 
 abstract public class NodeImpl extends AbstractNode {
-    
+
     protected static NodeInfo[] emptyArray = new NodeInfo[0];
 
     protected ParentNodeImpl parent;
@@ -38,7 +38,7 @@ abstract public class NodeImpl extends AbstractNode {
     * implements the javax.xml.transform.Source interface, allowing a node to be
     * used directly as the Source of a transformation
     */
-    
+
     public void setSystemId(String uri) {
         // overridden in DocumentImpl and ElementImpl
         getParent().setSystemId(uri);
@@ -50,7 +50,7 @@ abstract public class NodeImpl extends AbstractNode {
     * same node in the tree.
     */
 
-    public boolean isSameNode(NodeInfo other) {
+    public boolean isSameNodeInfo(NodeInfo other) {
         // default implementation: differs for attribute and namespace nodes
         return this==other;
     }
@@ -58,7 +58,7 @@ abstract public class NodeImpl extends AbstractNode {
 	/**
 	* Get the nameCode of the node. This is used to locate the name in the NamePool
 	*/
-	
+
 	public int getNameCode() {
 		// default implementation: return -1 for an unnamed node
 		return -1;
@@ -68,7 +68,7 @@ abstract public class NodeImpl extends AbstractNode {
 	* Get the fingerprint of the node. This is used to compare whether two nodes
 	* have equivalent names. Return -1 for a node with no name.
 	*/
-	
+
 	public int getFingerprint() {
 	    int nameCode = getNameCode();
 	    if (nameCode==-1) return -1;
@@ -78,7 +78,7 @@ abstract public class NodeImpl extends AbstractNode {
     /**
     * Get a character string that uniquely identifies this node within this document
     * (The calling code will prepend a document identifier)
-    * @return a string. 
+    * @return a string.
     */
 
     public String generateId() {
@@ -127,7 +127,7 @@ abstract public class NodeImpl extends AbstractNode {
             }
             prev = prev.getPreviousInDocument();
         }
-        
+
     }
 
     /**
@@ -138,7 +138,7 @@ abstract public class NodeImpl extends AbstractNode {
     * node, or 0 if they are the same node. (In this case, isSameNode() will always
     * return true, and the two nodes will produce the same result for generateId())
     */
-    
+
     public final int compareOrder(NodeInfo other) {
         long a = getSequenceNumber();
         long b = ((NodeImpl)other).getSequenceNumber();
@@ -150,7 +150,7 @@ abstract public class NodeImpl extends AbstractNode {
 	/**
 	* Get the NamePool
 	*/
-	
+
 	public NamePool getNamePool() {
 		return getDocumentRoot().getNamePool();
 	}
@@ -181,7 +181,7 @@ abstract public class NodeImpl extends AbstractNode {
     }
 
     /**
-    * Get the name of this node, following the DOM rules 
+    * Get the name of this node, following the DOM rules
     * @return The name of the node. For an element this is the element name, for an attribute
     * it is the attribute name, as a QName. Other node types return conventional names such
     * as "#text" or "#comment"
@@ -192,11 +192,11 @@ abstract public class NodeImpl extends AbstractNode {
     //    return getDisplayName();
     //}
 
-    
+
     /**
     * Get the display name of this node. For elements and attributes this is [prefix:]localname.
     * For unnamed nodes, it is an empty string.
-    * @return The display name of this node. 
+    * @return The display name of this node.
     * For a node with no name, return an empty string.
     */
 
@@ -207,8 +207,8 @@ abstract public class NodeImpl extends AbstractNode {
     }
 
     /**
-    * Get the local name of this node. 
-    * @return The local name of this node. 
+    * Get the local name of this node.
+    * @return The local name of this node.
     * For a node with no name, return an empty string.
     */
 
@@ -217,7 +217,7 @@ abstract public class NodeImpl extends AbstractNode {
     	if (nameCode == -1) return "";
         return getNamePool().getLocalName(nameCode);
     }
-    
+
     /**
     * Get the line number of the node within its source document entity
     */
@@ -268,7 +268,7 @@ abstract public class NodeImpl extends AbstractNode {
     * @return The previous sibling node. Returns null if the current node is the first
     * child of its parent.
     */
-    
+
     public Node getPreviousSibling()  {
         return parent.getNthChild(index-1);
     }
@@ -282,7 +282,7 @@ abstract public class NodeImpl extends AbstractNode {
 
     public Node getNextSibling()  {
         return parent.getNthChild(index+1);
-    }    
+    }
 
     /**
     * Get first child - default implementation used for leaf nodes
@@ -314,25 +314,25 @@ abstract public class NodeImpl extends AbstractNode {
     public AxisEnumeration getEnumeration(
                                         byte axisNumber,
                                         NodeTest nodeTest) {
-                                            
+
         switch (axisNumber) {
             case Axis.ANCESTOR:
                  return new AncestorEnumeration(this, nodeTest, false);
-                 
-            case Axis.ANCESTOR_OR_SELF: 
+
+            case Axis.ANCESTOR_OR_SELF:
                  return new AncestorEnumeration(this, nodeTest, true);
-                 
+
             case Axis.ATTRIBUTE:
                  if (this.getNodeType()!=ELEMENT) return EmptyEnumeration.getInstance();
                  return new AttributeEnumeration(this, nodeTest);
-                 
+
             case Axis.CHILD:
                  if (this instanceof ParentNodeImpl) {
                     return ((ParentNodeImpl)this).enumerateChildren(nodeTest);
                  } else {
                     return EmptyEnumeration.getInstance();
                  }
-                 
+
             case Axis.DESCENDANT:
                 if (getNodeType()==ROOT &&
                         nodeTest instanceof NameTest &&
@@ -344,55 +344,55 @@ abstract public class NodeImpl extends AbstractNode {
                 } else {
                     return EmptyEnumeration.getInstance();
                 }
-                 
+
             case Axis.DESCENDANT_OR_SELF:
                  return new DescendantEnumeration(this, nodeTest, true);
-                 
+
             case Axis.FOLLOWING:
                  return new FollowingEnumeration(this, nodeTest);
-                 
+
             case Axis.FOLLOWING_SIBLING:
                  return new FollowingSiblingEnumeration(this, nodeTest);
-                 
+
             case Axis.NAMESPACE:
                  if (this.getNodeType()!=ELEMENT) return EmptyEnumeration.getInstance();
                  return new NamespaceEnumeration(this, nodeTest);
-                    
+
             case Axis.PARENT:
                  NodeInfo parent = (NodeInfo)getParentNode();
                  if (parent==null) return EmptyEnumeration.getInstance();
                  if (nodeTest.matches(parent)) return new SingletonEnumeration(parent);
                  return EmptyEnumeration.getInstance();
-                 
+
             case Axis.PRECEDING:
                  return new PrecedingEnumeration(this, nodeTest);
-                 
+
             case Axis.PRECEDING_SIBLING:
                  return new PrecedingSiblingEnumeration(this, nodeTest);
-                 
+
             case Axis.SELF:
                  if (nodeTest.matches(this)) return new SingletonEnumeration(this);
                  return EmptyEnumeration.getInstance();
-                 
+
             case Axis.PRECEDING_OR_ANCESTOR:
                  return new PrecedingOrAncestorEnumeration(this, nodeTest);
-                 
+
             default:
                  throw new IllegalArgumentException("Unknown axis number " + axisNumber);
         }
-    }            
+    }
 
     /**
      * Returns whether this node (if it is an element) has any attributes.
-     * @return <code>true</code> if this node has any attributes, 
+     * @return <code>true</code> if this node has any attributes,
      *   <code>false</code> otherwise.
      * @since DOM Level 2
      */
-    
+
     public boolean hasAttributes() {
         return false;
     }
-    
+
     /**
      * Find the value of a given attribute of this node. <BR>
      * This method is defined on all nodes to meet XSL requirements, but for nodes
@@ -418,16 +418,16 @@ abstract public class NodeImpl extends AbstractNode {
     //public String getAttributeValue( String name ) {
     //    return null;
     //}
-    
+
     /**
     * Get the value of a given attribute of this node
     * @param fingerprint The fingerprint of the attribute name
     * @return the attribute value if it exists or null if not
     */
-    
+
     public String getAttributeValue(int fingerprint) {
     	return null;
-    }    
+    }
 
     /**
      * Get the outermost element.
@@ -435,7 +435,7 @@ abstract public class NodeImpl extends AbstractNode {
      * not well-formed, this returns the last element child of the root if there is one, otherwise
      * null.
      */
-     
+
     public Element getDocumentElement() {
         return ((DocumentImpl)getDocumentRoot()).getDocumentElement();
 
@@ -539,11 +539,11 @@ abstract public class NodeImpl extends AbstractNode {
     //}
 
     /**
-     * Return a <code>NodeList</code> that contains all children of this node. If 
-     * there are no children, this is a <code>NodeList</code> containing no 
+     * Return a <code>NodeList</code> that contains all children of this node. If
+     * there are no children, this is a <code>NodeList</code> containing no
      * nodes.
      */
-     
+
     public NodeList getChildNodes() {
         try {
             return new NodeSetExtent(EmptyEnumeration.getInstance(), null);
@@ -553,25 +553,25 @@ abstract public class NodeImpl extends AbstractNode {
     }
 
     /**
-     * Return a <code>NamedNodeMap</code> containing the attributes of this node (if 
+     * Return a <code>NamedNodeMap</code> containing the attributes of this node (if
      * it is an <code>Element</code> ) or <code>null</code> otherwise. (DOM method)
      */
-     
+
     public NamedNodeMap getAttributes() {
         // default implementation
         return null;
     }
 
 
-  
+
     /**
      * Determine whether the node has any children.
-     * @return  <code>true</code> if the node has any children, 
+     * @return  <code>true</code> if the node has any children,
      *   <code>false</code> if the node has no children.
      */
-     
+
     public boolean hasChildNodes() {
-        return getFirstChild() != null;         
+        return getFirstChild() != null;
     }
 
 }
@@ -579,18 +579,18 @@ abstract public class NodeImpl extends AbstractNode {
 //
 // The contents of this file are subject to the Mozilla Public License Version 1.0 (the "License");
 // you may not use this file except in compliance with the License. You may obtain a copy of the
-// License at http://www.mozilla.org/MPL/ 
+// License at http://www.mozilla.org/MPL/
 //
 // Software distributed under the License is distributed on an "AS IS" basis,
 // WITHOUT WARRANTY OF ANY KIND, either express or implied.
-// See the License for the specific language governing rights and limitations under the License. 
+// See the License for the specific language governing rights and limitations under the License.
 //
-// The Original Code is: all this file. 
+// The Original Code is: all this file.
 //
 // The Initial Developer of the Original Code is
-// Michael Kay of International Computers Limited (mhkay@iclway.co.uk).
+// Michael Kay
 //
-// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved. 
+// Portions created by (your name) are Copyright (C) (your legal entity). All Rights Reserved.
 //
-// Contributor(s): none. 
+// Contributor(s): none.
 //
